# -*- coding: utf-8 -*-
from collections import defaultdict
import errno
import functools
import inspect
import json
import math
import os
from socket import error as SocketError

from cdecimal import *
from delorean import Delorean, epoch
import termcolor as tc
import requests
from retrying import Retrying
from requests_futures.sessions import FuturesSession

from gryphon.lib.logger import get_logger
from gryphon.lib.models.datum import DatumRecorder
from gryphon.lib.slacker import Slacker
from gryphon.lib.session import get_a_redis_connection
import gryphon.lib.metrics.quote as quote_lib
from gryphon.lib.money import Money
from gryphon.lib.exchange.exchange_order import Order
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exceptions import *


logger = get_logger(__name__)

LOG_LINE_LIMIT = 100


class Exchange(object):
    def __init__(self, session=None):
        self.withdrawal_fee = Money('0', 'BTC')
        self.btc_credit_limit = Money('0', 'BTC')
        self.price_decimal_precision = None
        self.volume_decimal_precision = None
        self.volume_currency = 'BTC'

        if session:
            self.session = session
        else:
            self.session = FuturesSession(max_workers=10)
    
    @property
    def redis(self):
        if hasattr(self, '_redis'):
            return self._redis
        else:
            self._redis = get_a_redis_connection()
            return self._redis

    def load_env(self, key):
        return str(os.environ[key])

    ###### Request Methods ######

    def req(self, req_method, url, **kwargs):
        """
        Create a requests-future request.
        """

        # If the exchange has a base_url set for it's API we can send in only the
        # path to the API endpoint we're trying to reach.
        if url[:4] != 'http':
            url = self.base_url + url

        try:
            no_auth = kwargs['no_auth']
            del kwargs['no_auth']
        except KeyError:
            no_auth = False

        if not no_auth:
            self.auth_request(req_method, url, kwargs)

        req_method = req_method.lower()

        if req_method == 'post':
            future = self.session.post(url, **kwargs)
        elif req_method == 'get':
            future = self.session.get(url, **kwargs)
        elif req_method == 'delete':
            future = self.session.delete(url, **kwargs)
        else:
            raise ValueError('%s method not supported' % req_method)

        return future

    def resp(self, req):
        """
        Get the response from a requests-futures object.
        """

        # Get the function name that called us.
        try:
            stack = inspect.stack()
            api_method = stack[1][3]

            if api_method == 'resp':
                api_method = stack[2][3]

            api_method = api_method.replace('_resp', '')
        except IndexError:
            api_method = 'unknown'

        try:
            response = req.result()
            log_line = response.text

            if len(log_line) > LOG_LINE_LIMIT:
                log_line = log_line[:LOG_LINE_LIMIT]
                log_line += '...'

            logger.debug(u'[%s] API %s - %s' % (
                self.friendly_name,
                api_method,
                log_line,
            ))

            data = response.json(parse_float=Decimal)
        except ValueError: # includes JSONDecodeError
            raise ExchangeAPIFailureException(self, response)
        except (requests.exceptions.ConnectionError,
                    requests.exceptions.ChunkedEncodingError):
            raise ExchangeAPIFailureException(self)
        except SocketError as e:
            if e.errno == errno.ECONNRESET:
                raise ExchangeAPIFailureException(self)
            else:
                raise e # Not error we are looking for

        return data

    def from_const(self, mode):
        if mode == Consts.BID:
            return self.bid_string
        elif mode == Consts.ASK:
            return self.ask_string
        else:
            raise ValueError('mode must be a Const')

    def to_const(self, mode):
        if mode == self.bid_string:
            return Consts.BID
        elif mode == self.ask_string:
            return Consts.ASK
        else:
            raise ValueError(
                'mode must be one of %s/%s' % (self.bid_string, self.ask_string),
            )
    
    ###### Helper Methods ######
    
    def send_btc_to_exchange(self, exchange, amount):
        destination_address = exchange.deposit_address()

        logger.info('Sending %s from %s to %s (%s)' % (
            amount,
            self.name,
            exchange.name,
            destination_address,
        ))

        slacker = Slacker('#balancer','balancer', icon_emoji=':zap:')
        slacker.notify('Sending %s from %s to %s (%s)' % (
            amount,
            self.name,
            exchange.name,
            destination_address,
        ))

        result = self.withdraw_bitcoin(destination_address, amount)

        return (
            destination_address,
            result.get('tx'),
            result.get('exchange_withdrawal_id'),
        )

    def cached_orderbook(self, volume_limit=None):
        key = '%s_orderbook' % self.name.lower()
        raw_redis_result = self.redis.get(key)

        try:
            redis_result = json.loads(raw_redis_result)

            orderbook_age = Delorean().epoch - float(redis_result['timestamp'])

            if orderbook_age >= 10:
                raise CachedOrderbookFailure(
                    self,
                    'Cached orderbook more than 10 seconds old',
                )

            # Emerald Havoc sets the timestamp when it gets a new orderbook
            time_fetched = redis_result['timestamp']
            orderbook = redis_result[self.name]

            parsed_orderbook = self.parse_orderbook(
                orderbook,
                volume_limit,
                cached_orders=True,
            )

            parsed_orderbook['time_fetched'] = time_fetched

            return parsed_orderbook
        except CachedOrderbookFailure:
            # let the 'more than 10 seconds old' exception bubble up
            raise
        except Exception as e:
            raise CachedOrderbookFailure(self, 'Unable to parse cached orderbook')

    def cancel_all_open_orders(self):
        [self.cancel_order(o['id']) for o in self.open_orders()]

    def remove_our_orders(self, order_book, open_orders, only_flag=False):
        """
        TODO add flagging back in.
        """
        open_volume_by_price = defaultdict(int)

        for o in open_orders:
            open_volume_by_price[o['price'].amount] += o['volume_remaining']

        for order in order_book['bids'] + order_book['asks']:
            open_volume = open_volume_by_price.get(
                order.price.amount, Money('0', 'BTC'),
            )

            if open_volume > 0:
                volume_to_remove = min(open_volume, order.volume)
                if only_flag:
                    order.status = Order.FLAGGED
                else:
                    order.volume -= volume_to_remove
                open_volume -= volume_to_remove

        # Remove 0-volume orders.
        order_book['bids'] = [o for o in order_book['bids'] if o.volume > 0]
        order_book['asks'] = [o for o in order_book['asks'] if o.volume > 0]

    def price_quote(self, mode, volume):
        req = self.price_quote_req()
        return self.price_quote_resp(req, mode, volume)

    def price_quote_req(self):
        """
        Returns an expected total cost to buy the given bitcoin volume
        and also the price the bid should be made at.
        """
        return [self.open_orders_req(), self.get_order_book_req()]

    def price_quote_resp(self, reqs, mode, volume):
        if not isinstance(volume, Money) or volume.currency != self.volume_currency:
            raise TypeError('Volume must be %s' % self.volume_currency)

        open_orders = self.open_orders_resp(reqs[0])

        # We need this because liquidity gets removed when we remove our own orders
        # below.
        open_volume = sum([o['volume_remaining'] for o in open_orders])
        order_book = self.get_order_book_resp(reqs[1], volume + open_volume)

        self.remove_our_orders(order_book, open_orders)

        return quote_lib.price_quote_from_orderbook(order_book, mode, volume)

    def create_trade_at_market_price(self, mode, volume):
        price_quote = self.price_quote(mode, volume)

        logger.info(
            u'Quote for %s of %s is %s, place order at %s',
            mode,
            volume,
            price_quote['total_price'],
            price_quote['price_for_order'],
        )

        response = self.create_trade(mode, volume, price_quote['price_for_order'])

        if response.get('success'):
            response['price'] = price_quote['price_for_order']
            return response
        else:
            return None

    def buy_bitcoin(self, volume):
        return self.create_trade_at_market_price(Consts.BID, volume)

    def sell_bitcoin(self, volume):
        return self.create_trade_at_market_price(Consts.ASK, volume)

    def order_details(self, order_id):
        result = self.multi_order_details([order_id])

        if order_id in result:
            return result[order_id]
        else:
            raise ExchangeAPIErrorException(self, 'Order not found')

    def exchange_data(self, db):
        from gryphon.lib.exchange.exchange_factory import make_exchange_data_from_key
        return make_exchange_data_from_key(self.name, db)

    ###### Common Exchange Methods ######

    def auth_request(req_method, url, request_args):
        raise NotImplementedError

    def apply_fee(self, order):
        from gryphon.lib.exchange.exchange_order import Order

        if order.type == Order.ASK:
            order.price += (order.price * self.fee)
        elif order.type == Order.BID:
            order.price -= (order.price * self.fee)

    def balance(self):
        req = self.balance_req()
        return self.balance_resp(req)

    def balance_req(self):
        raise NotImplementedError

    def balance_resp(self, req):
        """
        -> Balance object
        """
        raise NotImplementedError


    def parse_cached_order(self, order):
        price = Money(order[0], self.currency)
        volume = Money(order[1], self.volume_currency)
        return (price, volume)

    def parse_any_order(self, order, is_cached_order=False):
        if is_cached_order:
            price, volume = self.parse_cached_order(order)
        else:
            price, volume = self.parse_order(order)

        return price, volume

    def parse_orderbook(self, raw_orderbook, volume_limit=None, price_limit=None, cached_orders=False):
        """
        Returns a dictionary containing a sorted list of asks and a sorted list of bids.
        """

        if cached_orders:
            raw_bids = raw_orderbook['bids']
            raw_asks = raw_orderbook['asks']
            sort_key = lambda o: float(o[0])
        else:
            raw_bids = self._get_raw_bids(raw_orderbook)
            raw_asks = self._get_raw_asks(raw_orderbook)
            sort_key = self._orderbook_sort_key

        raw_bids.sort(key = sort_key, reverse=True)
        raw_asks.sort(key = sort_key)

        bids = []
        asks = []

        total_volume = Money(0, self.volume_currency)
        top_bid_price, __ = self.parse_any_order(raw_bids[0], cached_orders)

        for bid in raw_bids:
            bid_price, bid_volume = self.parse_any_order(bid, cached_orders)

            if price_limit:
                bid_threshold = top_bid_price - price_limit
                if bid_price < bid_threshold:
                    break

            bids.append(Order(bid_price, bid_volume, self, Order.BID))

            total_volume += bid_volume
            if volume_limit and total_volume > volume_limit:
                break

        total_volume = Money(0, self.volume_currency)
        top_ask_price, __ = self.parse_any_order(raw_asks[0], cached_orders)

        for ask in raw_asks:
            ask_price, ask_volume = self.parse_any_order(ask, cached_orders)

            if price_limit:
                ask_threshold = top_ask_price + price_limit
                if ask_price > ask_threshold:
                    break

            asks.append(Order(ask_price, ask_volume, self, Order.ASK))

            total_volume += ask_volume
            if volume_limit and total_volume > volume_limit:
                break

        return {'bids':bids, 'asks':asks}

    def ticker(self, verify=True):
        req = self.ticker_req(verify=verify)
        return self.ticker_resp(req)

    def ticker_req(self, verify=True):
        raise NotImplementedError

    def ticker_resp(self, req):
        """
        -> {
            high: Money,
            low: Money,
            last: Money,
            volume: Money
        }
        """
        raise NotImplementedError

    def get_order_book(self, volume_limit=None, verify=True):
        req = self.get_order_book_req(verify=verify)
        return self.get_order_book_resp(req, volume_limit)

    def get_order_book_req(self, verify=True):
        if self.use_cached_orderbook:
            return 'CACHED_ORDERBOOK'

        return self._get_order_book_req(verify=verify)

    def get_order_book_resp(self, req, volume_limit=None):
        if req == 'CACHED_ORDERBOOK':
            return self.cached_orderbook(volume_limit)
        else:
            raw_orderbook = self._get_order_book_resp(req)
            fetched_orderbook = self.parse_orderbook(raw_orderbook, volume_limit)
            fetched_orderbook['time_fetched'] = Delorean().epoch

            return fetched_orderbook

    def _get_order_book_req(self, verify=True):
        raise NotImplementedError

    def _get_order_book_resp(self, req):
        """Get the raw orderbook from an exchange"""
        return self.resp(req)

    def _get_raw_bids(self, raw_orderbook):
        """
        The default behaviour in this ant _get_raw_asks works for about 70% of the
        exchanges, the rest override them.
        """
        return raw_orderbook['bids']

    def _get_raw_asks(self, raw_orderbook):
        return raw_orderbook['asks']

    @property
    def _orderbook_sort_key(self):
        return lambda order: float(order[0])

    def parse_order(self, order):
        price = Money(order[0], self.currency)
        volume = Money(order[1], self.volume_currency)

        return (price, volume)

    def create_trade(self, mode, volume, price, is_market_order=False):
        req = self.create_trade_req(mode, volume, price, is_market_order)
        return self.create_trade_resp(req)

    def create_trade_req(self, mode, volume, price, is_market_order=False):
        raise NotImplementedError
        
    def create_trade_resp(self, req):
        """
        -> {success:
            order_id:
            error:?
            }
        """
        raise NotImplementedError

    def open_orders(self):
        req = self.open_orders_req()
        return self.open_orders_resp(req)

    def open_orders_req(self):
        raise NotImplementedError

    def open_orders_resp(self, req):
        """
          -> {
              mode:
              id:
              price:
              volume:
          }
        """
        raise NotImplementedError

    def multi_order_status_req(self):
        return self.open_orders_req()

    def multi_order_status_resp(self, req, order_ids):
        order_ids = [str(o) for o in order_ids]

        open_orders = self.open_orders_resp(req)
        open_order_ids = [o['id'] for o in open_orders]

        orders = {}

        for order_id in order_ids:
            if order_id in open_order_ids:
                orders[order_id] = {'status': 'open'}
            else:
                orders[order_id] = {'status': 'filled'}

        return orders

    def multi_order_details(self, order_ids):
        """
        This is a strange, magical function. We need to handle exchanges where we send
        order_ids in the request [Bitfinex] e.g. 
            r = multi_order_details_req(order_ids)
            multi_order_details_resp(r)
        ...as well as exchanges where we send order_ids in the response (filtering after
        the fact) [Bitstamp]. e.g.
            r = multi_order_details_req()
            multi_order_details_resp(r, order_ids)
        ...and even some strange exchanges where we send it in both the request and the
        response [Kraken].
        """
        try:
            req = self.multi_order_details_req()
        except TypeError: # The method expected parameters which we didn't send
            req = self.multi_order_details_req(order_ids)

        try:
            return self.multi_order_details_resp(req)
        except TypeError: # The method expected parameters which we didn't send
            return self.multi_order_details_resp(req, order_ids)

    def multi_order_details_req(self):
        raise NotImplementedError

    def multi_order_details_resp(self, req, order_ids):
        """
        -> {
            time_created:
            type:
            btc_total:
            fiat_total:
            trades:
        }
        """
        raise NotImplementedError

    def cancel_order(self, order_id):
        req = self.cancel_order_req(order_id)
        return self.cancel_order_resp(req)

    def cancel_order_req(self, order_id):
        raise NotImplementedError

    def cancel_order_resp(self, req):
        """
        -> {success:
            error:
            }
        """
        raise NotImplementedError

    def fiat_deposit_fee(self, deposit_amount):
        return Money(0, self.currency)

    def fiat_withdrawal_fee(self, deposit_amount):
        return Money(0, self.currency)

    def deposit_address(self):
        """
        Override exchange-specific api methods with ENV vars.
        """
        env_key = '%s_DEPOSIT_ADDRESS' % self.name
        return os.environ[env_key]

    def deposit_address_req(self):
        raise NotImplementedError

    def deposit_address_resp(self, req):
        raise NotImplementedError

    def withdraw_bitcoin(self, address, bitcoin_volume):
        req = self.withdraw_bitcoin_req(address, bitcoin_volume)
        return self.withdraw_bitcoin_resp(req)

    def withdraw_bitcoin_req(self, address, bitcoin_volume):
        raise NotImplementedError

    def withdraw_bitcoin_resp(self, req):
        raise NotImplementedError

    def process_db_balance_for_audit(self, db_balance):
        return db_balance

    def pre_audit(self, exchange_data):
        pass

    def audit(self, skip_recent=0):
        raise NotImplementedError

    def maker_taker_fee_difference(self):
        """
        The fee difference between market and limit orders on this exchange.
        Defaults to zero (no difference).
        """
        if self.market_order_fee != None and self.limit_order_fee != None:
            return self.market_order_fee - self.limit_order_fee
        else:
            return Decimal('0')




BUILTIN_ORDERBOOK_CONFIGS = {
    'bchbtc': {
          'price_currency': 'BTC',
          'volume_currency': 'BCH',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'BCH-BTC',
          'trading_pairs': 'bitstamp_bch_btc',
          'DEFAULT_WIDTH': '0.01',
    },
    'bchusd': {
          'price_currency': 'USD',
          'volume_currency': 'BCH',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'BCH-USD',
          'trading_pairs': 'bitstamp_bch_usd',
          'DEFAULT_WIDTH': '100',
    },
    'bcheur': {
          'price_currency': 'EUR',
          'volume_currency': 'BCH',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'BCH-EUR',
          'trading_pairs': 'bitstamp_bch_eur',
          'DEFAULT_WIDTH': '20',
    },
    'btccad': {
          'price_currency': 'CAD',
          'volume_currency': 'BTC',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'BTC-CAD',
          'trading_pairs': 'kraken_btc_cad',
          'DEFAULT_WIDTH': '100',
    },
    'btceur': {
          'price_currency': 'EUR',
          'volume_currency': 'BTC',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'BTC-EUR',
          'trading_pairs': 'bitstamp_btc_eur, kraken_btc_eur',
          'DEFAULT_WIDTH': '100',
    },
    'btcusd': {
          'price_currency': 'USD',
          'volume_currency': 'BTC',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'BTC-USD',
          'trading_pairs': 'bitstamp_btc_usd, coinbase_btc_usd, gemini_btc_usd, itbit_btc_usd, kraken_btc_usd',
          'DEFAULT_WIDTH': '50',
    },
    'ethbtc': {
          'price_currency': 'BTC',
          'volume_currency': 'ETH',
          'infinitisimal': 0.00001,
          'infinity': 100,
          'zero': 0,
          'pair_name': 'ETH-BTC',
          'trading_pairs': 'gemini_eth_btc, bitstamp_eth_btc, poloniex_eth_btc',
          'DEFAULT_WIDTH': '0.02',
    },
    'etheur': {
          'price_currency': 'EUR',
          'volume_currency': 'ETH',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'ETH-EUR',
          'trading_pairs': 'bitstamp_eth_eur',
          'DEFAULT_WIDTH': '90',
    },
    'ethusd': {
          'price_currency': 'USD',
          'volume_currency': 'ETH',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'ETH-USD',
          'trading_pairs': 'bitstamp_eth_usd, gemini_eth_usd',
          'DEFAULT_WIDTH': '10',
    },
}





# -*- coding: utf-8 -*-
import base64
from collections import OrderedDict
import hashlib
import hmac
import os
import time
import urllib

from cdecimal import Decimal
from delorean import Delorean
from more_itertools import chunked

from gryphon.lib.exchange import exceptions
from gryphon.lib.exchange import order_types
from gryphon.lib.exchange.exchange_api_wrapper import ExchangeAPIWrapper
from gryphon.lib.logger import get_logger
from gryphon.lib.models.exchange import Balance
from gryphon.lib.money import Money

logger = get_logger(__name__)


class LedgerSizeException(Exception):
    pass


class KrakenBTCEURExchange(ExchangeAPIWrapper):
    def __init__(self, session=None, configuration=None):
        super(KrakenBTCEURExchange, self).__init__(session, configuration)

        # Immutable properties and endpoints.
        self.name = u'KRAKEN_BTC_EUR'
        self.friendly_name = u'Kraken BTC-EUR'
        self.base_url = 'https://api.kraken.com/0'
        self.price_decimal_precision = 1
        self.currency = u'EUR'
        self.volume_currency = u'BTC'
        self.bid_string = 'buy'
        self.ask_string = 'sell'

        self.orderbook_depth = 100000

        # Configurables with defaults.
        self.market_order_fee = Decimal('0.0014')
        self.limit_order_fee = Decimal('0.0004')
        self.fee = self.market_order_fee
        self.fiat_balance_tolerance = Money('0.0001', 'EUR')
        self.volume_balance_tolerance = Money('0.00000001', 'BTC')
        self.min_order_size = Money('0.002', 'BTC')
        self.max_tick_speed = 5
        self.use_cached_orderbook = False

        # TODO: Unclear if these should be configurable or not.
        self.withdrawal_fee = Money('0.001', 'BTC')
        self.btc_credit_limit = Money('0', 'BTC')

        if configuration:
            self.configure(configuration)

    @property
    def pair(self):
        return self.construct_pair(self.currency)

    def req(self, req_method, url, **kwargs):
        req = super(KrakenBTCEURExchange, self).req(req_method, url, **kwargs)

        if not kwargs.get('no_auth'):
            time.sleep(0.1)  # For nonces.

        return req

    def resp(self, req):
        response = super(KrakenBTCEURExchange, self).resp(req)

        if response.get('error'):
            errors_string = str(response['error'])

            if 'Insufficient funds' in errors_string:
                raise exceptions.InsufficientFundsError()
            elif 'Unknown order' in errors_string:
                raise exceptions.CancelOrderNotFoundError()
            elif 'Invalid nonce' in errors_string:
                raise exceptions.NonceError()
            else:
                raise exceptions.ExchangeAPIErrorException(self, errors_string)

        try:
            return response['result']
        except KeyError:
            raise exceptions.ExchangeAPIFailureException(self, response)

    def get_trades_info(self, trade_ids):
        req = self.get_trades_info_req(trade_ids)
        return self.get_trades_info_resp(req)

    def get_trades_info_req(self, trade_ids):
        reqs = []

        # /private/QueryTrades only accepts 20 ids at a time.
        for trade_ids_chunk in chunked(trade_ids, 20):
            payload = {
                'txid': ','.join(trade_ids_chunk),
            }

            req = self.req('post', '/private/QueryTrades', data=payload)
            reqs.append(req)

        return reqs

    def get_trades_info_resp(self, reqs):
        result = {}

        for req in reqs:
            result.update(self.resp(req))

        return result

    def get_trades_info_from_ledger(self, trade_ids, order_open_timestamp, order_close_timestamp):
        """
        Check the ledger entries to get accurate numbers for how much our balance was
        changed.

        The ledger is Kraken's only real source of truth, the trades/order endpoints
        lie to us.
        """

        # We add a 0.1s buffer to make sure we get entries right on the boundary
        # timestamps.
        ledger_start = order_open_timestamp - Decimal('0.1')
        # We add a 1s buffer because it takes Kraken a bit of time to write to the
        # ledger.
        ledger_end = order_close_timestamp + Decimal('1')

        entries = self.get_ledger_entries(
            start=ledger_start,
            end=ledger_end,
        )

        trades_info = {}

        for trade_id in trade_ids:
            trades_info[trade_id] = {
                'btc': Money.loads('BTC 0'),
                'btc_fee': Money.loads('BTC 0'),
                'fiat': Money(0, self.currency),
                'fiat_fee': Money(0, self.currency),
            }

        for ledger_id, entry in entries.iteritems():
            trade_id = entry['refid']

            if trade_id not in trade_ids:
                continue

            amount = Decimal(entry['amount'])

            if entry['type'] == 'credit':
                # Credit ledger entries show up when we dip into our line of credit.
                # They have opposite signs, and need to be included along side the
                # trade ledger entries to get accurate trade amounts.
                amount = -amount
            elif entry['type'] == 'trade':
                pass
            else:
                raise exceptions.ExchangeAPIErrorException(
                    self,
                    'Unexpected ledger entry type "%s"' % entry['type'],
                )

            currency = self.convert_from_kraken_currency(entry['asset'])

            if currency == 'BTC':
                trades_info[trade_id]['btc'] += Money(amount, 'BTC')
                trades_info[trade_id]['btc_fee'] += Money(entry['fee'], 'BTC')
            else:
                trades_info[trade_id]['fiat'] += Money(amount, currency)
                trades_info[trade_id]['fiat_fee'] += Money(entry['fee'], currency)

            # There are multiple ledger entries per trade, but they should all be going
            #through at the same time, so we can just take the timestamp of the last
            # one.
            trades_info[trade_id]['time'] = entry['time']

        return trades_info

    def ledger_entry(self, ledger_id):
        req = self.ledger_entry_req(ledger_id)
        return self.ledger_entry_resp(req)

    def ledger_entry_req(self, ledger_id):
        payload = {
            'id': ledger_id,
        }

        return self.req('post', '/private/QueryLedgers', data=payload)

    def ledger_entry_resp(self, req):
        return self.resp(req)

    def get_ledger_entries(self, type=None, start=None, end=None):
        try:
            req = self.get_ledger_entries_req(type, start, end)
            return self.get_ledger_entries_resp(req)
        except LedgerSizeException:
            # If there were too many ledger entries for this time period, split the
            # period in half and run each one seperately.
            # It's a bit of a hack but this is a rare edge case.
            mid = (start + end) / 2
            first_half = self.get_ledger_entries(type, start, mid)
            second_half = self.get_ledger_entries(type, mid, end)

            full_ledger = first_half
            full_ledger.update(second_half)

            return full_ledger

    def get_ledger_entries_req(self, type=None, start=None, end=None):
        payload = {}

        if type:
            payload['type'] = type

        if start:
            payload['start'] = '%.3f' % start

        if end:
            payload['end'] = '%.3f' % end

        return self.req('post', '/private/Ledgers', data=payload)

    def get_ledger_entries_resp(self, req):
        response = self.resp(req)
        count = int(response['count'])

        entries = response['ledger']

        if count > len(entries):
            raise LedgerSizeException('More entries than fit on one page of results')

        return entries

    def closed_orders(self, start=None, end=None, offset=0):
        req = self.closed_orders_req(start, end, offset)
        return self.closed_orders_resp(req)

    def closed_orders_req(self, start=None, end=None, offset=0):
        payload = {}

        if start:
            payload['start'] = start

        if end:
            payload['end'] = end

        if offset:
            payload['ofs'] = str(offset)

        return self.req('post', '/private/ClosedOrders', data=payload)

    def closed_orders_resp(self, req):
        response = self.resp(req)
        count = int(response['count'])
        closed_orders = []

        for order_id, raw_order in response['closed'].iteritems():
            raw_order['order_id'] = order_id
            closed_orders.append(raw_order)

        return count, closed_orders

    ######## Common Exchange Methods ########

    def load_creds(self):
        try:
            self.api_key
            self.secret
        except AttributeError:
            self.api_key = self._load_env('KRAKEN_BTC_EUR_API_KEY')
            self.secret = self._load_env('KRAKEN_BTC_EUR_API_SECRET')

    def auth_request(self, req_method, url, request_args):
        """
        This modifies the request_args.
        """
        self.load_creds()

        endpoint = url.replace(self.base_url, '')
        endpoint = '/0' + endpoint

        nonce = unicode(int(round(time.time() * 1000)))

        try:
            payload = request_args['data']
        except KeyError:
            payload = request_args['data'] = {}

        payload.update({
            'nonce': nonce,
        })

        post_data = urllib.urlencode(payload)
        message = endpoint + hashlib.sha256(nonce + post_data).digest()
        sig = base64.b64encode(hmac.new(base64.b64decode(self.secret), message, hashlib.sha512).digest())

        try:
            headers = request_args['headers']
        except KeyError:
            headers = request_args['headers'] = {}

        headers.update({
            'API-Key': self.api_key,
            'API-Sign': sig,
        })

    def get_balance_req(self):
        return self.req('post', '/private/BalanceEx')

    def get_balance_resp(self, req):
        response = self.resp(req)

        total_balance = Balance()

        # We only want to load the balances for the pair we trade on this account
        # (BTC + self.currency).  For example we have a small USD balance in our main
        # EUR Kraken account which we want to ignore. Including it in the balance breaks
        # some other parts of our system which expect to only find a single fiat
        # currency.
        supported_currencies = ['BTC', self.currency]

        for currency in supported_currencies:
            kraken_currency = self.convert_to_kraken_currency(currency)

            balance_amount = Decimal(response[kraken_currency]['balance'])
            credit_amount = Decimal(response[kraken_currency].get('credit_used', 0))

            balance = Money(balance_amount, currency)
            credit = Money(credit_amount, currency)

            total_balance += (balance - credit)

        return total_balance

    def get_ticker_req(self, verify=True):
        url = '/public/Ticker?pair=%s' % self.pair
        return self.req('get', url, no_auth=True, verify=verify)

    def get_ticker_resp(self, req):
        response = self.resp(req)
        ticker = response[self.pair]

        return {
            'high': Money(ticker['h'][1], self.currency),
            'low': Money(ticker['l'][1], self.currency),
            'last': Money(ticker['c'][0], self.currency),
            'volume': Money(ticker['v'][1], 'BTC'),
        }

    def _get_orderbook_from_api_req(self, verify=True):
        url = '/public/Depth?pair=%s&count=%s' % (self.pair, self.orderbook_depth)
        return self.req('get', url, no_auth=True, verify=verify)

    def _get_raw_bids(self, raw_orderbook):
        return raw_orderbook[self.pair]['bids']

    def _get_raw_asks(self, raw_orderbook):
        return raw_orderbook[self.pair]['asks']

    def place_order_req(self, mode, volume, price, order_type=order_types.LIMIT_ORDER):
        mode = self._order_mode_from_const(mode)

        try:
            payload = {
                'pair': self.pair,
                'type': mode,
                'ordertype': 'limit',
                'price': unicode(price.amount),
                'volume': unicode(volume.amount),
            }

        except AttributeError:
            raise TypeError('volume and price must be Money objects')

        return self.req('post', '/private/AddOrder', data=payload)

    def place_order_resp(self, req):
        response = self.resp(req)

        try:
            return {'success': True, 'order_id': unicode(response['txid'][0])}
        except KeyError:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'response does not contain an order_id',
            )

    def get_open_orders_req(self):
        payload = {}
        return self.req('post', '/private/OpenOrders', data=payload)

    def get_open_orders_resp(self, req):
        response = self.resp(req)
        open_orders = []

        try:
            raw_open_orders = response['open']

            for order_id, raw_order in raw_open_orders.iteritems():
                if raw_order['status'] == 'open':
                    mode = self._order_mode_to_const(raw_order['descr']['type'])
                    volume = Money(raw_order['vol'], 'BTC')
                    volume_executed = Money(raw_order['vol_exec'], 'BTC')
                    price = Money(raw_order['descr']['price'], self.currency)

                    order = {
                        'mode': mode,
                        'id': order_id,
                        'price': price,
                        'volume_remaining': volume - volume_executed,
                    }

                    open_orders.append(order)
        except KeyError:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'Open orders format incorrect',
            )

        return open_orders

    def get_order_details(self, order_id):
        req = self.get_order_details_req(order_id)
        return self.get_order_details_resp(req, order_id)

    def get_order_details_req(self, order_id):
        return self.get_multi_order_details_req([order_id])

    def get_order_details_resp(self, req, order_id):
        return self.get_multi_order_details_resp(req, [order_id])[order_id]
            
    def get_multi_order_details(self, order_ids):
        req = self.get_multi_order_details_req(order_ids)
        return self.get_multi_order_details_resp(req, order_ids)
 
    def get_multi_order_details_req(self, order_ids):
        order_ids = [unicode(o) for o in order_ids]

        payload = {
            'trades': True,
            'txid': ','.join(order_ids),
        }

        return self.req('post', '/private/QueryOrders', data=payload)

    def get_multi_order_details_resp(self, req, order_ids):
        multi_trades = self.resp(req)
        data = {}

        for order_id in order_ids:
            total_fiat = Money('0', self.currency)
            total_btc = Money('0', 'BTC')
            our_trades = []

            if order_id in multi_trades:
                order = multi_trades[order_id]
                trade_ids = order.get('trades', [])

                if trade_ids:
                    opentm = order['opentm']

                    # Partially-executed orders haven't "closed" yet so they don't
                    # have a closetm. We only need the already executed trades, so we
                    # end the interval at NOW().
                    if 'closetm' in order:
                        closetm = order['closetm']
                    else:
                        closetm = Decimal(Delorean().epoch)

                    trades = self.get_trades_info_from_ledger(
                        trade_ids,
                        opentm,
                        closetm,
                    )

                    for t_id, t in trades.iteritems():
                        fiat = abs(t['fiat'])
                        btc = abs(t['btc'])

                        if t['btc_fee'] and t['fiat_fee']:
                            raise exceptions.ExchangeAPIErrorException(
                                self,
                                '#%s charged fees in both fiat (%s) and BTC (%s)' % (
                                order_id,
                                t['fiat_fee'],
                                t['btc_fee'],
                            ))
                        elif t['btc_fee']:
                            fee = t['btc_fee']
                        else:
                            fee = t['fiat_fee']

                        total_fiat += fiat
                        total_btc += btc

                        our_trades.append({
                            'time': int(t['time']),
                            'trade_id': unicode(t_id),
                            'fee': fee,
                            'btc': btc,
                            'fiat': fiat,
                        })

            data[order_id] = {
                'time_created': int(order['opentm']),
                'type': self._order_mode_to_const(order['descr']['type']),
                'btc_total': total_btc,
                'fiat_total': total_fiat,
                'trades': our_trades,
            }

        return data

    def cancel_order_req(self, order_id):
        payload = {
            'txid': unicode(order_id),
        }

        return self.req('post', '/private/CancelOrder', data=payload)

    def cancel_order_resp(self, req):
        response = self.resp(req)

        if response.get('count', 0) > 0:
            return {'success': True}
        else:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'cancelled count should be > 0',
            )

    def withdraw_crypto_req(self, address, volume):
        if not isinstance(address, basestring):
            raise TypeError('Withdrawal address must be a string')

        if self.volume_currency != 'BTC':
            raise TypeError('Kraken withdrawals are only supported for BTC right now.')

        if not isinstance(volume, Money) or volume.currency != self.volume_currency:
            raise TypeError('Withdrawal volume must be in %s' % self.volume_currency)

        # The Kraken API only takes names which are mapped to address. The mapping can
        # be adjusted here: https://www.kraken.com/u/funding/withdraw?asset=XXBT
        # Since our system passes around addresses, we search through the ENV vars to
        # find the corresponding exchange name, which we then pass to Kraken.
        deposit_addresses = {
            name: addr
            for name, addr in os.environ.iteritems() if '_DEPOSIT_ADDRESS' in name
        }

        address_to_name_map = {
            addr: name.replace('_DEPOSIT_ADDRESS', '')
            for name, addr in deposit_addresses.iteritems()
        }

        try:
            destination_exchange_name = address_to_name_map[address]
        except KeyError:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'Could not find matching exchange for %s' % address,
            )

        volume += self.withdrawal_fee

        payload = {
            'asset': 'XXBT',
            'key': destination_exchange_name,
            'amount': volume.amount,
        }

        return self.req('post', '/private/Withdraw', data=payload)

    def withdraw_crypto_resp(self, req):
        response = self.resp(req)
        return response

    def get_order_audit_data(self, skip_recent=0, lookback_hours=2):
        """
        Returns an OrderedDict of order ids mapped to their filled volume (including
        only orders that have some trades).
        """
        raw_orders = []
        start = Delorean().last_hour(lookback_hours).epoch

        offset = 0
        count, raw_chunk = self.closed_orders(start)
        raw_orders += raw_chunk
        offset += len(raw_chunk)

        while offset < count:
            logger.info('%s orders fetched, %s remaining' % (offset, count - offset))
            _, raw_chunk = self.closed_orders(start, offset=offset)
            raw_orders += raw_chunk
            offset += len(raw_chunk)

        raw_orders = raw_orders[skip_recent:]

        orders = OrderedDict()

        for raw_order in raw_orders:
            volume_executed = Money(raw_order['vol_exec'], 'BTC')

            if volume_executed > 0:
                order_id = raw_order['order_id']
                orders[order_id] = volume_executed

        return orders

    def fiat_deposit_fee(self, deposit_amount):
        return Money('5', 'EUR')

    @classmethod
    def construct_pair(cls, fiat_currency):
        btc_currency_code = cls.convert_to_kraken_currency('BTC')
        fiat_currency_code = cls.convert_to_kraken_currency(fiat_currency)

        return btc_currency_code + fiat_currency_code

    @classmethod
    def convert_from_kraken_currency(cls, kraken_currency):
        if kraken_currency == 'XXBT':
            currency = 'BTC'
        elif kraken_currency[0] == 'Z':
            currency = kraken_currency[1:]

        if currency not in Money.CURRENCIES:
            raise ValueError('invalid currency value: \'%s\'' % currency)

        return currency

    @classmethod
    def convert_to_kraken_currency(cls, currency):
        if currency == 'BTC':
            kraken_currency = 'XXBT'
        else:
            kraken_currency = 'Z' + currency

        return kraken_currency



import pyximport; pyximport.install()

import os
import unittest
import sure
from mock import patch

from gryphon.lib.money import Money
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.models.exchange import Exchange, Position
from gryphon.lib.models.trade import Trade
from gryphon.lib.models.order import Order

import logging
logger = logging.getLogger(__name__)

class TestOrder(unittest.TestCase):
    def mockExchange(self):
        exchange = Exchange("TEST")
        exchange.currency = "USD"
        return exchange

    def setUp(self):
        self.price = Money(100, "USD")
        self.volume = Money(1, "BTC")
        exchange = self.mockExchange()
        self.order = Order("Test", Consts.BID, self.volume, self.price, exchange, None)

        price = Money(50, "USD")
        fee = Money(1, "USD")
        volume = Money("0.5", "BTC")
        t = Trade(Consts.BID, price, fee, volume, None, None)
        self.order.trades.append(t)

        price = Money(25, "USD")
        fee = Money(1, "USD")
        volume = Money("0.25", "BTC")
        t = Trade(Consts.BID, price, fee, volume, None, None)
        self.order.trades.append(t)

    def tearDown(self):
        pass

    def test_position(self):
        self.order.position.should.have.key('USD')
        self.order.position.should.have.key('BTC')
        self.order.position['USD'].should.equal(Money(-77, "USD"))
        self.order.position['BTC'].should.equal(Money("0.75", "BTC"))

    def test_position_change(self):
        old_position = Position()
        delta = self.order.position_change(old_position)
        delta['USD'].should.equal(Money(-77, "USD"))
        delta['BTC'].should.equal(Money("0.75", "BTC"))

    def test_reverse_position_change(self):
        old_position = self.order.position
        empty_data = {'btc_total': Money.loads("BTC 0"),
            'fiat_total': Money.loads("USD 0"),
            'time_created': 1431712831,
            'trades': [],
            'type': u'ASK'}

        self.order.was_partially_eaten(empty_data)
        delta = self.order.position_change(old_position)
        delta['USD'].should.equal(Money(77, "USD"))
        delta['BTC'].should.equal(Money("-0.75", "BTC"))

    def test_reversal(self):
        # delta from 0 to order position
        delta = self.order.position_change(Position())
        old_position = self.order.position
        self.order.reverse()
        # delta back from order position to 0
        reverse_delta = self.order.position_change(old_position)
        reverse_delta.should.equal(-delta)



"""
Tests for gryphon.execution.auditing.
"""

import pyximport; pyximport.install()
import gryphon.lib; gryphon.lib.prepare()

import unittest
import sure

from gryphon.execution.lib import auditing
from gryphon.lib.money import Money
from gryphon.lib.models.exchange import Balance


class TestAuditing(unittest.TestCase):
    def test_trivial(self):
        db_balance = Balance()
        ledger_balance = Balance()

        auditing.balance_equality(db_balance, ledger_balance).should.equal(True)

    def test_symmetric_equal(self):
        db_balance = Balance({'USD': Money('50', 'USD'), 'BTC': Money('0', 'BTC')})
        ledger_balance = Balance({'USD': Money('50', 'USD'), 'BTC': Money('0', 'BTC')})

        auditing.balance_equality(db_balance, ledger_balance).should.equal(True)

    def test_symmetric_not_equal(self):
        db_balance = Balance({'USD': Money('50', 'USD'), 'BTC': Money('0', 'BTC')})
        ledger_balance = Balance({'USD': Money('51', 'USD'), 'BTC': Money('0', 'BTC')})

        auditing.balance_equality(db_balance, ledger_balance).should.equal(False)

    def test_symmetric_very_not_equal(self):
        db_balance = Balance({'USD': Money('50', 'USD'), 'BTC': Money('0', 'BTC')})

        ledger_balance = Balance({
            'USD': Money('-60', 'USD'),
            'BTC': Money('100', 'BTC'),
        })

        auditing.balance_equality(db_balance, ledger_balance).should.equal(False)

    def test_complex_symmetric_equal(self):
        db_balance = Balance({
            'USD': Money('50', 'USD'),
            'CAD': Money('10000', 'CAD'),
            'ETH': Money('0', 'ETH'),
            'BTC': Money('0', 'BTC'),
        })

        ledger_balance = Balance({
            'USD': Money('50', 'USD'),
            'CAD': Money('10000', 'CAD'),
            'ETH': Money('0', 'ETH'),
            'BTC': Money('0', 'BTC'),
        })

        auditing.balance_equality(db_balance, ledger_balance).should.equal(True)

    def test_complex_symmetric_not_equal(self):
        db_balance = Balance({
            'USD': Money('50', 'USD'),
            'CAD': Money('10000', 'CAD'),
            'ETH': Money('0', 'ETH'),
            'BTC': Money('0', 'BTC'),
        })

        ledger_balance = Balance({
            'USD': Money('50', 'USD'),
            'CAD': Money('10000', 'CAD'),
            'ETH': Money('-1000', 'ETH'),
            'BTC': Money('-5000', 'BTC'),
        })

        auditing.balance_equality(db_balance, ledger_balance).should.equal(False)

    def test_asymmetric_equal(self):
        db_balance = Balance({'USD': Money('50', 'USD'), 'BTC': Money('0', 'BTC')})
        ledger_balance = Balance({'USD': Money('50', 'USD')})

        auditing.balance_equality(db_balance, ledger_balance).should.equal(True)

    def test_asymmetric_not_equal(self):
        db_balance = Balance({'USD': Money('50', 'USD'), 'BTC': Money('0', 'BTC')})
        ledger_balance = Balance({'USD': Money('51', 'USD')})

        auditing.balance_equality(db_balance, ledger_balance).should.equal(False)

    def test_asymmetric_very_not_equal(self):
        db_balance = Balance({'USD': Money('50', 'USD'), 'BTC': Money('0', 'BTC')})
        ledger_balance = Balance({'USD': Money('-10000', 'USD')})

        auditing.balance_equality(db_balance, ledger_balance).should.equal(False)

    def test_complex_asymmetric_equal(self):
        db_balance = Balance({
            'USD': Money('50', 'USD'),
            'CAD': Money('10000', 'CAD'),
            'ETH': Money('0', 'ETH'),
            'BTC': Money('0', 'BTC'),
        })

        ledger_balance = Balance({
            'USD': Money('50', 'USD'),
            'CAD': Money('10000', 'CAD'),
        })

        auditing.balance_equality(db_balance, ledger_balance).should.equal(True)

    def test_complex_asymmetric_not_equal(self):
        db_balance = Balance({
            'USD': Money('50', 'USD'),
            'CAD': Money('10000', 'CAD'),
            'ETH': Money('0', 'ETH'),
            'BTC': Money('0', 'BTC'),
        })

        ledger_balance = Balance({
            'USD': Money('50', 'USD'),
            'CAD': Money('-10000', 'CAD'),
        })

        auditing.balance_equality(db_balance, ledger_balance).should.equal(False)





import pyximport; pyximport.install()

import os
import unittest

from cdecimal import Decimal

from gryphon.lib.metrics import orderbook_strength
from gryphon.lib.exchange.exchange_order import Order as ExchangeOrder
from gryphon.lib.models.order import Order
from gryphon.lib.money import Money


class TestLiquidityFunction(unittest.TestCase):
    def setUp(self):
        self.empty_book = {
            'bids': [],
            'asks': [],
        }

        self.exchange = None

        bid = ExchangeOrder(
            Money('249', 'USD'), 
            Money('10', 'BTC'),
            self.exchange,
            Order.BID,
        )
            
        ask = ExchangeOrder(
            Money('251', 'USD'), 
            Money('10', 'BTC'),
            self.exchange,
            Order.ASK,
        )

        self.basic_book = {
            'bids': [bid],
            'asks': [ask],
        }

        bid2 = ExchangeOrder(
            Money('239', 'USD'),
            Money('10', 'BTC'),
            self.exchange,
            Order.BID,
        )

        ask2 = ExchangeOrder(
            Money('261', 'USD'),
            Money('100', 'BTC'),
            self.exchange,
            Order.ASK,
        )

        self.bigger_book = {
            'bids': [bid, bid2],
            'asks': [ask, ask2],
        }

    def test_strength_basic(self):
        bid_strength = orderbook_strength.orderbook_strength_at_slippage(
            self.basic_book,
            Order.BID,
            Money('2', 'USD'),
        )

        ask_strength = orderbook_strength.orderbook_strength_at_slippage(
            self.basic_book,
            Order.ASK,
            Money('2', 'USD'),
        )

        bid_strength.should.equal(Decimal('10'))
        ask_strength.should.equal(Decimal('10'))

    def test_strength_normal(self):
        """
        This orderbook is:
            [261, 100]
            [251, 10]
            .
            [249, 10]
            [239, 10]
        """
        bid_strength = orderbook_strength.orderbook_strength_at_slippage(
            self.bigger_book,
            Order.BID,
            Money('20', 'USD'),
        )

        ask_strength = orderbook_strength.orderbook_strength_at_slippage(
            self.bigger_book,
            Order.ASK,
            Money('20', 'USD'),
        )

        bid_strength.should.equal(Decimal('20'))
        ask_strength.should.equal(Decimal('110'))




import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("Console")
from utils.balancer import proportions, account_value_limits, bitcoin_rebalance
from gryphon.lib.money import Money

def sanity_bitcoin_rebalance(db, exchange_data, other_exchange_data):
    logger.info('SANITY TEST FOR BITCOIN REBALANCER')
    logger.info('!!!!!!!!!!!!!!!!! USE ONLY WITH TEST / STAGIN DB. THIS WILL LOCK THE DB !!!!!!!!!!!!!!!!!!!')
    
    # Test 1 - exchange_data will not send bitcoin
    exchange_data.balance['BTC'] = Money(0, 'BTC')
    bitcoin_rebalance(db, exchange_data, False)
    logger.info('EXPECTED TEST RESULT %s will be the exchange most in need. Wont send to self.\n\n' % exchange_data.name)
    
    # Test 2 - exchange_data will send to other exchange_data
    exchange_data.balance['BTC'] = Money(50000, 'BTC')
    other_exchange_data.balance['BTC'] = Money(0, 'BTC')
    bitcoin_rebalance(db, exchange_data, False)
    logger.info('EXPECTED TEST RESULT %s will send to %s or another exchange with 0 BTC.\n\n' % (exchange_data.name, other_exchange_data.name))
    
    # Test 3 - not send due to account limit
    exchange_data.balance['BTC'] = Money(50000, 'BTC')
    other_exchange_data.balance['BTC'] = Money(500, 'BTC')
    other_exchange_data.balance['USD'] += account_value_limits[other_exchange_data.name] 
    bitcoin_rebalance(db, exchange_data, False)
    logger.info('EXPECTED TEST RESULT %s will not send to %s because it is above its limit. Should send to another exchange. \n\n' % (exchange_data.name, other_exchange_data.name))
    
    #Test 4 - exchanges are only 9btc apart
    exchange_data.balance['BTC'] = Money(70, 'BTC')
    exchange_data.balance['USD'] = account_value_limits[exchange_data.name]/2
    other_exchange_data.balance['BTC'] = Money(61, 'BTC')
    other_exchange_data.balance['USD'] = account_value_limits[other_exchange_data.name]/2
    bitcoin_rebalance(db, exchange_data, False)
    logger.info('EXPECTED TEST RESULTS: %s will not send to %s becuase the exchanges are close together. This test aint perfect.\n\n' % (exchange_data.name, other_exchange_data.name) )
    
    #Test 5 - exchanges are only 15btc apart
    exchange_data.balance['BTC'] = Money(75, 'BTC')
    exchange_data.balance['USD'] = account_value_limits[exchange_data.name]/2
    other_exchange_data.balance['BTC'] = Money(55, 'BTC')
    other_exchange_data.balance['USD'] = account_value_limits[other_exchange_data.name]/2
    bitcoin_rebalance(db, exchange_data, False)
    logger.info('EXPECTED TEST RESULTS: %s will  send to %s becuase the exchanges are close together but far enough apart. This test aint perfect.\n\n' % (exchange_data.name, other_exchange_data.name) )
    
    #Test 6 - exchanges has more than its target in full balance but less than its target in its balance, it should not send
    exchange_data.balance['BTC'] = Money(10, 'BTC')
    other_exchange_data.balance['BTC'] = Money(110, 'BTC')
    other_exchange_data.record_withdrawal(exchange_data, Money(100, 'BTC'), None, None, None)
    bitcoin_rebalance(db, exchange_data, False)
    logger.info('EXPECTED TEST RESULT: %s will not send to %s becuase it doesnt have enough in its balance, even though it has lots in its full_balance'  % (exchange_data.name, other_exchange_data.name))
    
    
    
    



# We need to use absolute_import so we can load the coinbase pip module without
# conflicting with our filename of coinbase.py.
from __future__ import absolute_import
import os
import base64
from collections import OrderedDict
import hashlib
import hmac
import json
import time
import urllib

from cdecimal import Decimal
import coinbase.client

from gryphon.lib.exchange import exceptions
from gryphon.lib.exchange import order_types
from gryphon.lib.exchange.exchange_api_wrapper import ExchangeAPIWrapper
from gryphon.lib.logger import get_logger
from gryphon.lib.models.exchange import Balance
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.money import Money
from gryphon.lib.time_parsing import parse

logger = get_logger(__name__)


class CoinbaseBTCUSDExchange(ExchangeAPIWrapper):
    def __init__(self, session=None, configuration=None):
        super(CoinbaseBTCUSDExchange, self).__init__(session)

        self.name = u'COINBASE_BTC_USD'
        self.friendly_name = u'Coinbase BTC-USD'
        self.base_url = 'https://api.gdax.com'
        self.currency = 'USD'
        self.bid_string = 'buy'
        self.ask_string = 'sell'
        self.product_id = 'BTC-USD'
        self.price_decimal_precision = 2

        # Configurables with defaults.
        self.market_order_fee = Decimal('0.0022') # Updated by Gareth on 2016-9-20
        self.limit_order_fee = Decimal('0.001')
        self.fee = self.market_order_fee
        self.fiat_balance_tolerance = Money('0.0001', 'USD')
        self.volume_balance_tolerance = Money('0.00000001', 'BTC')
        self.max_tick_speed = 1
        self.min_order_size = Money('0.001', 'BTC')
        self.use_cached_orderbook = False

        if configuration:
            self.configure(configuration)

    def req(self, req_method, url, **kwargs):
        try:
            kwargs['data'] = json.dumps(kwargs['data'])
        except KeyError:
            kwargs['data'] = '{}'

        kwargs['headers'] = {'Content-Type': 'application/json'}
        req = super(CoinbaseBTCUSDExchange, self).req(req_method, url, **kwargs)

        return req

    def resp(self, req):
        response = req.result()

        try:
            data = response.json(parse_float=Decimal)
        except ValueError:
            raise exceptions.ExchangeAPIFailureException(self, response)

        headers = response.headers

        if response.status_code < 200 or response.status_code >= 300:
            try:
                error_string = data['message']
            except KeyError:
                error_string = str(data)

            error_string = error_string.lower()

            if 'notfound' in error_string:
                raise exceptions.NoEffectOrderCancelledError()
            elif ('order already done' in error_string or
                    'order not found' in error_string):
                raise exceptions.CancelOrderNotFoundError()
            elif 'order size is too small' in error_string:
                raise exceptions.MinimumOrderSizeError()
            elif 'insufficient funds' in error_string:
                raise exceptions.InsufficientFundsError()
            # These errors occur randomly (usually when Coinbase under heavy load).
            # We want to return an ExchangeAPIFailureException so that requests get
            # retried.
            elif ('request timestamp expired' in error_string or
                    'internal server error' in error_string):
                raise exceptions.ExchangeAPIFailureException(self, response)
            else:
                raise exceptions.ExchangeAPIErrorException(self, error_string)
        else:
            return data, headers

    def pagination_cursors(self, headers):
        return {
            'prev_cursor': headers.get('CB-BEFORE'),
            'next_cursor': headers.get('CB-AFTER')
        }

    def ledger(self, account_id=None, return_pagination=False, next_cursor=None):
        req = self.ledger_req(account_id, next_cursor=next_cursor)
        return self.ledger_resp(req, return_pagination=return_pagination)

    def ledger_req(self, account_id=None, next_cursor=None):
        if not account_id:
            account_id = self.load_fiat_account_id()

        params = {}

        if next_cursor:
            params['after'] = next_cursor

        endpoint = '/accounts/%s/ledger' % account_id

        if params:
            endpoint += '?%s' % urllib.urlencode(params)

        return self.req('get', endpoint)

    def ledger_resp(self, req, return_pagination=False):
        response, headers = self.resp(req)

        if return_pagination:
            return response, self.pagination_cursors(headers)
        else:
            return response

    def _get_recent_trades(self, order_id=None, return_pagination=False, next_cursor=None):
        req = self._get_recent_trades_req(order_id, next_cursor=next_cursor)
        return self._get_recent_trades_resp(req, return_pagination=return_pagination)

    def _get_recent_trades_req(self, order_id=None, next_cursor=None):
        params = {
            'product_id': self.product_id
        }

        if order_id:
            params['order_id'] = order_id

        if next_cursor:
            params['after'] = next_cursor

        endpoint = '/fills?%s' % urllib.urlencode(params)

        return self.req('get', endpoint)

    def _get_recent_trades_resp(self, req, return_pagination=False):
        response, headers = self.resp(req)
        trades = []

        for trade in response:
            if not trade['settled']:
                continue

            price = Money(trade['price'], self.currency)
            size = Money(trade['size'], 'BTC')
            fiat = price * size.amount
            our_type = self._order_mode_to_const(trade['side'])

            # Strange bug here, delorean isn't parsing the trailing Z on the created_at
            # date correctly.
            trade_dict = {
                'time': int(parse(trade['created_at'][:-1]).epoch),
                'trade_id': str(trade['trade_id']),
                'order_id': str(trade['order_id']),
                'btc': size,
                'fiat': fiat,
                'fee': Money(trade['fee'], self.currency),
                'type': our_type,
            }

            trades.append(trade_dict)

        if return_pagination:
            return trades, self.pagination_cursors(headers)
        else:
            return trades

    def transfer(self, transfer_type, btc_amount, coinbase_account_id):
        req = self.transfer_req(transfer_type, btc_amount, coinbase_account_id)
        return self.transfer_resp(req)

    def transfer_req(self, transfer_type, btc_amount, coinbase_account_id):
        payload = {
            'type': transfer_type,
            'amount': str(btc_amount.amount),
            'coinbase_account_id': coinbase_account_id
        }

        return self.req('post', '/transfers', data=payload)

    def transfer_resp(self, req):
        response, headers = self.resp(req)
        return response

    def load_wallet_creds(self):
        try:
            self.wallet_api_key
            self.wallet_api_secret
            self.wallet_id
        except AttributeError:
            self.wallet_api_key = self._load_env('COINBASE_BTC_USD_WALLET_API_KEY')
            self.wallet_api_secret = self._load_env('COINBASE_BTC_USD_WALLET_API_SECRET')
            self.wallet_id = self._load_env('COINBASE_BTC_USD_WALLET_ID')

    def load_fiat_account_id(self):
        return self._load_env('COINBASE_BTC_USD_FIAT_ACCOUNT_ID')

    @property
    def wallet_api_client(self):
        """API Client for the regular Coinbase Wallet (not the exchange API)"""
        if hasattr(self, '_wallet_api_client'):
            return self._wallet_api_client
        else:
            self.load_wallet_creds()
            self._wallet_api_client = coinbase.client.Client(
                self.wallet_api_key,
                self.wallet_api_secret,
            )

            return self._wallet_api_client

    def get_wallet(self):
        self.load_wallet_creds()
        return self.wallet_api_client.get_account(self.wallet_id)

    def transfer_wallet_balance_into_exchange(self):
        wallet = self.get_wallet()
        balance = Money(wallet.balance.amount, wallet.balance.currency)
        if balance > 0:
            self.transfer('deposit', balance, wallet.id)

    def check_for_daily_fee_rebate(self, exchange_data):
        recent_transactions = self.ledger()
        rebates = [tx for tx in recent_transactions if tx['type'] == 'rebate']

        for rebate in rebates:
            rebate_id = rebate['details']['rebate_id']

            # Searching transaction_details for the rebate_id is pretty janky but we
            # need to figure out a strategy for handling rebates anyways so this is
            # good enough for now.
            search_string = '%{}%'.format(rebate_id)
            existing_rebate_transaction = exchange_data.transactions\
                .filter(Transaction._transaction_details.like('%fee rebate%'))\
                .filter(Transaction._transaction_details.like(search_string))\
                .first()

            if existing_rebate_transaction:
                logger.debug('Found rebate, but it has already been recorded (%s)' % rebate_id)
                continue

            amount = Money(rebate['amount'], self.currency)

            logger.debug('Recording rebate for %s' % amount)

            transaction_details = {
                'notes': 'fee rebate',
                'rebate_id': rebate_id,
            }

            deposit = Transaction(
                Transaction.DEPOSIT,
                Transaction.IN_TRANSIT,
                amount,
                exchange_data,
                transaction_details,
            )
            # We deliberately leave the deposit IN_TRANSIT, and the next audit will see
            # the balance change and COMPLETE it through the regular deposit_landed
            # flow.

    # Common Exchange Methods

    def load_creds(self):
        try:
            self.api_key
            self.secret
            self.passphrase
        except AttributeError:
            self.api_key = self._load_env('COINBASE_BTC_USD_API_KEY')
            self.passphrase = self._load_env('COINBASE_BTC_USD_API_PASSPHRASE')
            self.secret = self._load_env('COINBASE_BTC_USD_API_SECRET')

    def auth_request(self, req_method, url, request_args):
        """
        This modifies request_args.
        """
        self.load_creds()

        req_method = req_method.upper()
        timestamp = unicode(int(round(time.time())))

        # This has already been dumped to json by req().
        body = request_args['data']

        endpoint = url.replace(self.base_url, '')

        data = timestamp + req_method + endpoint + body
        key = base64.b64decode(self.secret)
        sig = base64.b64encode(hmac.new(key, data, hashlib.sha256).digest())

        try:
            headers = request_args['headers']
        except KeyError:
            headers = request_args['headers'] = {}

        headers.update({
            'CB-ACCESS-KEY': self.api_key,
            'CB-ACCESS-SIGN': sig,
            'CB-ACCESS-PASSPHRASE': self.passphrase,
            'CB-ACCESS-TIMESTAMP': timestamp
        })

    def get_balance_req(self):
        return self.req('get', '/accounts')

    def get_balance_resp(self, req):
        response, headers = self.resp(req)

        balance = Balance()

        try:
            for account in response:
                if account['currency'] == 'BTC':
                    balance['BTC'] = Money(account['available'], 'BTC')
                elif account['currency'] == self.currency:
                    balance[self.currency] = Money(account['available'], self.currency)
        except KeyError:
            raise exceptions.ExchangeAPIErrorException(self, 'malformed response')

        return balance

    def get_ticker_req(self, verify=True):
        return self.req(
            'get',
            '/products/%s/stats' % self.product_id,
            no_auth=True,
            verify=verify,
        )

    def get_ticker_resp(self, req):
        response, headers = self.resp(req)

        return {
            'high': Money(response['high'], self.currency),
            'low': Money(response['low'], self.currency),
            'last': None,
            'volume': Money(response['volume'], 'BTC')
        }

    def _get_orderbook_from_api_req(self, verify=True):
        orderbook_url = '/products/%s/book?level=3' % self.product_id
        return self.req('get', orderbook_url, no_auth=True, verify=verify)

    def _get_orderbook_from_api_resp(self, req):
        response, headers = self.resp(req)
        return response

    def place_order_req(self, mode, volume, price, order_type=order_types.LIMIT_ORDER):
        mode = self._order_mode_from_const(mode)

        if price.currency != self.currency:
            raise ValueError('price must be in %s' % self.currency)
        if volume.currency != 'BTC':
            raise ValueError('volume must be in BTC')

        volume_str = '%.8f' % volume.amount
        price_str = '%.2f' % price.amount

        payload = {
            'product_id': self.product_id,
            'side': mode,
            'size': volume_str,
            'price': price_str,
        }

        if order_type == order_types.POST_ONLY:
            payload['post_only'] = True

        return self.req('post', '/orders', data=payload)

    def place_order_resp(self, req):
        response, headers = self.resp(req)
        order_id = response['id']

        return {'success': True, 'order_id': order_id}

    def get_open_orders_req(self):
        return self.req('get', '/orders')

    def get_open_orders_resp(self, req):
        raw_open_orders, headers = self.resp(req)
        open_orders = []

        for raw_order in raw_open_orders:
            mode = self._order_mode_to_const(raw_order['side'])
            volume = Money(raw_order['size'], 'BTC')
            volume_executed = Money(raw_order['filled_size'], 'BTC')
            volume_remaining = volume - volume_executed
            order = {
                'mode': mode,
                'id': str(raw_order['id']),
                'price': Money(raw_order['price'], self.currency),
                'volume': volume,
                'volume_remaining': volume_remaining
            }

            open_orders.append(order)

        return open_orders

    def get_order_details(self, order_id):
        req = self.get_order_details_req(order_id)
        return self.get_order_details_resp(req)

    def get_order_details_req(self, order_id):
        reqs = {}
        endpoint = '/orders/%s' % order_id
        reqs['order'] = (self.req('get', endpoint))

        # This kind of breaks the req/resp paradigm, but we need to get the results of
        # each batch before we know if we need to fetch another batch.
        trades = []

        trade_batch, pagination = self._get_recent_trades(
            order_id,
            return_pagination=True,
        )

        trades += trade_batch

        # Getting < 100 trades back means we have reached the end.
        while len(trade_batch) >= 100:
            time.sleep(1)

            trade_batch, pagination = self._get_recent_trades(
                order_id,
                return_pagination=True,
                next_cursor=pagination['next_cursor'],
            )

            trades += trade_batch

        # Instead of returning the requests, we return the results since we have them
        # already.
        reqs['trades'] = trades

        return reqs

    def get_order_details_resp(self, reqs):
        try:
            raw_order, headers = self.resp(reqs['order'])
        except exceptions.NoEffectOrderCancelledError:
            # Coinbase returns an API Error with the text "NotFound" in it when
            # querying orders that were cancelled with no trades, so we return an empty
            # order here if that is the response.
            result = {
                'time_created': None,
                'type': None,
                'btc_total': Money('0', self.volume_currency),
                'fiat_total': Money('0', self.currency),
                'trades': [],
            }

            return result

        # This already has the results (see above) so we don't need to .resp() it
        our_trades = reqs['trades']

        mode = self._order_mode_to_const(raw_order['side'])
        total_btc = Money(raw_order['filled_size'], 'BTC')
        time_created = int(parse(raw_order['created_at']).epoch)
        total_fiat = Money('0', self.currency)

        for t in our_trades:
            total_fiat += t['fiat']

        result = {
            'time_created': time_created,
            'type': mode,
            'btc_total': total_btc,
            'fiat_total': total_fiat,
            'trades': our_trades
        }

        return result

    def get_multi_order_details_req(self, order_ids):
        data = {}

        for oid in order_ids:
            data[oid] = self.get_order_details_req(oid)
            time.sleep(1) # 1 r/s rate limits :(

        return data

    def get_multi_order_details_resp(self, reqs):
        data = {}

        for oid in reqs:
            data[oid] = self.get_order_details_resp(reqs[oid])

        return data

    def cancel_order_req(self, order_id):
        endpoint = '/orders/%s' % order_id
        return self.req('delete', endpoint)

    def cancel_order_resp(self, req):
        # This is a weird one. It doesn't return JSON on success so we can't use
        # regular self.resp().
        response = req.result()

        if response.status_code == 200 and response.text == 'OK':
            return {'success': True}
        else:
            # This exception logic is more complicated than it appears.
            # If an order had no effect (no fills), cancelling it twice will
            # give back the 'notfound' response from coinbase, which raises
            # NoEffectOrderCancelledError, but the bots expect a double cancel
            # to raise CancelOrderNotFoundError, so here we catch the first
            # exception and raise the second. If the order has fllls on it,
            # the response from coinbase will raise CancelOrderNotFoundError
            # in self.resp naturally.
            try:
                resp = self.resp(req)
            except exceptions.NoEffectOrderCancelledError:
                raise exceptions.CancelOrderNotFoundError()

    def withdraw_crypto(self, address, volume):
        if not isinstance(address, basestring):
            raise TypeError('Withdrawal address must be a string')

        if not isinstance(volume, Money) or volume.currency != 'BTC':
            raise TypeError('Withdrawal volume must be in %s' % self.volume_currency)

        # Step 1: Withdraw from exchange to wallet.
        wallet = self.get_wallet()
        self.transfer('withdraw', volume, wallet.id)

        # Step 2: Send from wallet to address.
        response = wallet.send_money(address, str(volume.amount))
        tx = response['hsh']

        return {'success': True, 'tx': tx}

    def pre_audit(self, exchange_data):
        # As soon as deposits land in our wallet, we want to transfer them into the
        # exchange.
        # TODO: rehabilitate these.
        # self.transfer_wallet_balance_into_exchange()
        # self.check_for_daily_fee_rebate(exchange_data)
        pass

    def get_order_audit_data(self, skip_recent=0, lookback_pages=1):
        """
        Returns an OrderedDict of order ids mapped to their filled volume (only include
        orders that have some trades).
        """
        orders = OrderedDict()

        trades_to_audit, pagination = self._get_recent_trades(return_pagination=True)
        current_page = 1

        while current_page < lookback_pages:
            batch, pagination = self._get_recent_trades(
                next_cursor=pagination['next_cursor'],
                return_pagination=True,
            )

            trades_to_audit += batch
            current_page += 1

        # Trades from the same order aren't guaranteed to be next to each other, so we
        # need to group them.
        trades_to_audit.sort(key=lambda t: (t['time']), reverse=True)

        for trade in trades_to_audit:
            order_id = str(trade['order_id'])

            try:
                orders[order_id] += abs(trade['btc'])
            except KeyError:
                orders[order_id] = abs(trade['btc'])

        # Remove the oldest 2 orders, because its trades might be wrapped around a
        # page gap and this would give us an innacurate volume_filled number.
        # We need to remove 2 because there could be an ask and a bid.
        try:
            orders.popitem()
            orders.popitem()
        except KeyError:
            pass

        return orders

    def fiat_deposit_fee(self, deposit_amount):
        return Money('25', 'USD')

    def fiat_withdrawal_fee(self, withdrawal_amount):
        coinbase_fee = Money('40', 'USD')
        bmo_fee = Money('20', 'USD')

        return coinbase_fee + bmo_fee



# -*- coding: utf-8 -*-
from collections import OrderedDict
import hashlib
import hmac
import time

from cdecimal import *
from delorean import Delorean, parse, epoch

from base import *
from exceptions import *
from gryphon.lib.exchange.bitstamp_btc_usd import BitstampBTCUSDExchange
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.models.exchange import Balance
from gryphon.lib.money import Money

from gryphon.lib.logger import get_logger
logger = get_logger(__name__)


class BitstampBTCEURExchange(BitstampBTCUSDExchange):
    def __init__(self, session=None, configuration=None):
        super(BitstampBTCEURExchange, self).__init__(session)

        self.name = u'BITSTAMP_BTC_EUR'
        self.friendly_name = u'Bitstamp BTC-EUR'
        self.currency = u'EUR'
        self.volume_currency = 'BTC'

        self.fiat_balance_tolerance = Money('0.0001', 'EUR')
        self.volume_balance_tolerance = Money('0.00000001', 'BTC')
        self.min_order_size = Money('0.001', 'BTC')

        if configuration:
            self.configure(configuration)




import requests
import datetime
from delorean import Delorean
from cdecimal import Decimal
from gryphon.lib.logger import get_logger
logger = get_logger(__name__)

class BitcoinCharts(object):
    trades_base_url = 'http://api.bitcoincharts.com/v1/trades.csv?symbol=bitstampUSD&start='


    """
        This function seems to be off frequently by about 1000 bitcoins
        from the graphs bitcoincharts gives. This may be because they
        don't operate in UTC or could be something more sinister.
        For now it acts as a reasonable approximation.

        The control flow of this function is a little bit hard to 
        understand immediately. Essentially it continues requesting more
        trades from bitcoincharts until it sees one that is after the 'end'
        argument, but we have to guard for several edge cases, such as when
        we're requesting the most recent period of trades, in which case
        we will never hit the normal termination condition.
    """
    def get_bitstamp_volume_between_timestamps(self, start, end):
        volume = Decimal('0.0')
        last_processed_timestamp = start

        while last_processed_timestamp < end:
            request_url = self.trades_base_url + str(last_processed_timestamp)

            req = requests.get(request_url)

            if len(req.text) < 2:
                logger.info("Bad response from bitcoincharts")
                break

            lines = req.text.split('\n')

            for line in lines:
                tokens = line.split(',')

                trade_volume = float(tokens[2].strip())
                trade_timestamp = int(tokens[0])

                last_processed_timestamp = trade_timestamp

                if not last_processed_timestamp < end:
                    break
                else:
                    volume = volume + Decimal(trade_volume)
            
            # this catches the case that it was a single-line response
            # but that one line is a trade shortly before our 'end'
            if len(lines) <= 1:
                break

        return volume


    def get_24_hour_bitstamp_exchange_volume(self):
        start = int(Delorean().last_day().epoch)
        end = int(Delorean().epoch)

        volume = self.get_bitstamp_volume_between_timestamps(
            start,
            end,
        )

        return volume
        




"""Initial Migration

Revision ID: 4ee15d6375e
Revises: None
Create Date: 2019-03-18 15:38:42.643916

"""

# revision identifiers, used by Alembic.
revision = '4ee15d6375e'
down_revision = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import mysql

from gryphon.lib.models.exchange import JSONEncodedMoneyDict
sa.JSONEncodedMoneyDict = JSONEncodedMoneyDict

def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('event',
    sa.Column('unique_id', sa.Unicode(length=64), nullable=False),
    sa.Column('event_id', sa.Integer(), nullable=False),
    sa.Column('time_created', sa.DateTime(), nullable=False),
    sa.Column('exchange_name', sa.Unicode(length=256), nullable=False),
    sa.Column('event_type', sa.Unicode(length=256), nullable=False),
    sa.Column('data', sa.UnicodeText(length=2147483648), nullable=True),
    sa.PrimaryKeyConstraint('event_id')
    )
    op.create_table('exchange',
    sa.Column('exchange_id', sa.Integer(), nullable=False),
    sa.Column('unique_id', sa.Unicode(length=64), nullable=False),
    sa.Column('name', sa.Unicode(length=64), nullable=False),
    sa.Column('position', sa.JSONEncodedMoneyDict(), nullable=True),
    sa.Column('target', sa.JSONEncodedMoneyDict(), nullable=True),
    sa.Column('balance', sa.JSONEncodedMoneyDict(), nullable=True),
    sa.Column('multi_position_cache', sa.Numeric(precision=24, scale=14), nullable=True),
    sa.PrimaryKeyConstraint('exchange_id')
    )
    op.create_table('flag',
    sa.Column('key', sa.Unicode(length=64), nullable=False),
    sa.Column('value', sa.UnicodeText(length=2147483648), nullable=True),
    sa.PrimaryKeyConstraint('key')
    )
    op.create_table('liability',
    sa.Column('liability_id', sa.Integer(), nullable=False),
    sa.Column('unique_id', sa.Unicode(length=64), nullable=False),
    sa.Column('time_created', sa.DateTime(), nullable=False),
    sa.Column('amount', sa.Numeric(precision=24, scale=14), nullable=True),
    sa.Column('amount_currency', sa.Unicode(length=3), nullable=True),
    sa.Column('liability_type', sa.Unicode(length=64), nullable=True),
    sa.Column('entity_name', sa.Unicode(length=128), nullable=False),
    sa.Column('time_started', sa.DateTime(), nullable=True),
    sa.Column('time_repayed', sa.DateTime(), nullable=True),
    sa.Column('details', sa.UnicodeText(length=2147483648), nullable=True),
    sa.PrimaryKeyConstraint('liability_id')
    )
    op.create_table('order',
    sa.Column('order_id', sa.Integer(), nullable=False),
    sa.Column('status', sa.Unicode(length=32), nullable=False),
    sa.Column('order_type', sa.Unicode(length=64), nullable=True),
    sa.Column('unique_id', sa.Unicode(length=64), nullable=False),
    sa.Column('exchange_order_id', sa.Unicode(length=64), nullable=True),
    sa.Column('actor', sa.Unicode(length=64), nullable=True),
    sa.Column('exchange_rate', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('time_created', sa.DateTime(), nullable=False),
    sa.Column('time_executed', sa.DateTime(), nullable=True),
    sa.Column('exchange_name', sa.Unicode(length=64), nullable=True),
    sa.Column('price', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('price_currency', sa.Unicode(length=64), nullable=True),
    sa.Column('volume', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('volume_currency', sa.Unicode(length=64), nullable=True),
    sa.Column('fundamental_value', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('fundamental_value_currency', sa.Unicode(length=3), nullable=True),
    sa.Column('competitiveness', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('competitiveness_currency', sa.Unicode(length=3), nullable=True),
    sa.Column('spread', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('spread_currency', sa.Unicode(length=3), nullable=True),
    sa.PrimaryKeyConstraint('order_id')
    )
    op.create_table('result',
    sa.Column('result_id', sa.Integer(), nullable=False),
    sa.Column('ticks', sa.Integer(), nullable=False),
    sa.Column('unique_id', sa.Unicode(length=64), nullable=False),
    sa.Column('algorithm', sa.Unicode(length=64), nullable=False),
    sa.Column('batch', sa.Unicode(length=64), nullable=False),
    sa.Column('trading_volume', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('usd', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('btc', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('time_created', sa.DateTime(), nullable=False),
    sa.PrimaryKeyConstraint('result_id')
    )
    op.create_table('datum',
    sa.Column('unique_id', sa.Unicode(length=64), nullable=False),
    sa.Column('datum_id', sa.Integer(), nullable=False),
    sa.Column('time_created', sa.DateTime(), nullable=False),
    sa.Column('datum_type', sa.Unicode(length=256), nullable=False),
    sa.Column('numeric_value', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('string_value', sa.Unicode(length=256), nullable=True),
    sa.Column('meta_data', sa.UnicodeText(length=2147483648), nullable=True),
    sa.Column('order_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['order_id'], ['order.order_id'], ),
    sa.PrimaryKeyConstraint('datum_id')
    )
    op.create_table('result_trade',
    sa.Column('trade_id', sa.Integer(), nullable=False),
    sa.Column('price', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('price_currency', sa.Unicode(length=3), nullable=True),
    sa.Column('volume', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('volume_currency', sa.Unicode(length=3), nullable=True),
    sa.Column('tick', sa.Integer(), nullable=True),
    sa.Column('user_trade', sa.Unicode(length=64), nullable=True),
    sa.Column('result_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['result_id'], ['result.result_id'], ),
    sa.PrimaryKeyConstraint('trade_id')
    )
    op.create_table('transaction',
    sa.Column('transaction_id', sa.Integer(), nullable=False),
    sa.Column('transaction_type', sa.Unicode(length=64), nullable=True),
    sa.Column('transaction_status', sa.Unicode(length=64), nullable=True),
    sa.Column('unique_id', sa.Unicode(length=64), nullable=False),
    sa.Column('time_created', sa.DateTime(), nullable=False),
    sa.Column('time_completed', sa.DateTime(), nullable=True),
    sa.Column('amount', sa.Numeric(precision=24, scale=14), nullable=True),
    sa.Column('amount_currency', sa.Unicode(length=3), nullable=True),
    sa.Column('fee', sa.Numeric(precision=24, scale=14), nullable=True),
    sa.Column('fee_currency', sa.Unicode(length=3), nullable=True),
    sa.Column('transaction_details', sa.UnicodeText(length=2147483648), nullable=True),
    sa.Column('exchange_id', sa.Integer(), nullable=True),
    sa.Column('fee_buyback_transaction_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['exchange_id'], ['exchange.exchange_id'], ),
    sa.ForeignKeyConstraint(['fee_buyback_transaction_id'], ['transaction.transaction_id'], ),
    sa.PrimaryKeyConstraint('transaction_id')
    )
    op.create_table('trade',
    sa.Column('trade_id', sa.Integer(), nullable=False),
    sa.Column('trade_type', sa.Unicode(length=64), nullable=True),
    sa.Column('unique_id', sa.Unicode(length=64), nullable=False),
    sa.Column('exchange_trade_id', sa.Unicode(length=64), nullable=True),
    sa.Column('time_created', sa.DateTime(), nullable=False),
    sa.Column('fee', sa.Numeric(precision=24, scale=14), nullable=True),
    sa.Column('fee_currency', sa.Unicode(length=3), nullable=True),
    sa.Column('price', sa.Numeric(precision=24, scale=14), nullable=True),
    sa.Column('price_currency', sa.Unicode(length=3), nullable=True),
    sa.Column('volume', sa.Numeric(precision=24, scale=14), nullable=True),
    sa.Column('volume_currency', sa.Unicode(length=3), nullable=True),
    sa.Column('meta_data', sa.UnicodeText(length=2147483648), nullable=True),
    sa.Column('fee_buyback_transaction_id', sa.Integer(), nullable=True),
    sa.Column('order_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['fee_buyback_transaction_id'], ['transaction.transaction_id'], ),
    sa.ForeignKeyConstraint(['order_id'], ['order.order_id'], ),
    sa.PrimaryKeyConstraint('trade_id')
    )

    # Indices.
    op.create_index('idx_exchange_order_id_exchange_name', 'order', ['exchange_order_id', 'exchange_name'])
    op.create_index('idx_status', 'order', ['status'])
    op.create_index('time_created', 'order', ['time_created'])
    op.create_index('exchange_name', 'order', ['exchange_name'])

    op.create_index('time_created', 'datum', ['time_created'])

    op.create_index('time_created', 'event', ['time_created'])

    op.create_index('time_created', 'trade', ['time_created'])
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('trade')
    op.drop_table('transaction')
    op.drop_table('result_trade')
    op.drop_table('datum')
    op.drop_table('result')
    op.drop_table('order')
    op.drop_table('liability')
    op.drop_table('flag')
    op.drop_table('exchange')
    op.drop_table('event')
    ### end Alembic commands ###




def trade_diff(data_set):
    prices = [t.price for t in data_set]
    diffs = []
    for i in range(len(prices)):
        try:
            diffs.append(abs(prices[i+1] - prices[i]))
        except:
            pass
    return sum(diffs)/len(diffs)



"""
A small library that helps us calculate various positions. Of a trades list, of an
algorithm, of an exchange ledger. Lots of definitions of position around here.
"""

from sqlalchemy import func

from gryphon.lib.exchange.consts import Consts
from gryphon.lib.logger import get_logger
from gryphon.lib.models.exchange import Exchange as ExchangeData
from gryphon.lib.models.order import Order
from gryphon.lib.models.trade import Trade
from gryphon.lib.money import Money

logger = get_logger(__name__)


def fast_position(db, start_time=None, end_time=None, exchange_name=None, volume_currency='BTC', actor='Multi'):
    """
    Get the position of an exchange or strategy quickly using sql summing instead of
    loading every trade and doing it in python.

    TODO: We should extend this to not require an actor or volume currency.
    """
    query = db.query(func.sum(Trade._volume)).join(Order).filter(Order.actor == actor)

    if start_time:
        query = query.filter(Trade.time_created >= start_time)

    if end_time:
        query = query.filter(Trade.time_created < end_time)

    if exchange_name:
        query = query.filter(Order._exchange_name == exchange_name)

    raw_bid_volume = query.filter(Trade.trade_type == Consts.BID).scalar()
    raw_ask_volume = query.filter(Trade.trade_type == Consts.ASK).scalar()

    bid_volume = Money('0', volume_currency)

    if raw_bid_volume:
        bid_volume = Money(raw_bid_volume, volume_currency)

    ask_volume = Money('0', volume_currency)

    if raw_ask_volume:
        ask_volume = Money(raw_ask_volume, volume_currency)

    position = bid_volume - ask_volume

    return position


def cached_multi_position(db):
    """
    Get the multi strategy's position quickly by summing the cached multi position of
    all ExchangeData objects.
    """
    cached_position_sum = db\
        .query(func.sum(ExchangeData._multi_position_cache))\
        .filter(ExchangeData._multi_position_cache != None)\
        .scalar()

    cached_position = Money(cached_position_sum, 'BTC')

    # This is a hack to hardcode this, but it's a last line of defense against bad
    # position data. 250 is arbitrarily chosen as a higher bound for a strange-but-not-
    # buggy position. Our max position to date was 207BTC on the crazy okcoin 3-minute
    # tick day.
    max_sane_position = Money('250', 'BTC')

    if abs(cached_position) > max_sane_position:
        raise Exception('Insane position detected: %s' % cached_position)

    return cached_position


def position_delta(trades, price_currency=None, volume_currency='BTC'):
    """
    This function calculates our position across a list of trades for both the price
    and volume currencies. This is equivalent to calculating the total amount our
    balance in the two currencies have changed, and to the difference between the
    summation of ask and bid prices for the volume currency in that list.
    
    This function, though fairly simple, can be used in coordination with other
    functions to calculate realized or unrealized profit sets of trades. See:
    realized_pl and open_pl for examples.
    """

    if not price_currency:
        price_currency = price_currency_for_trades(trades)

    fiat_position = Money('0', price_currency)
    vol_position = Money('0', volume_currency)

    for trade in trades:
        if trade.trade_type == Consts.ASK:
            vol_position -= trade.volume
            fiat_position += trade.price_in_currency(price_currency)
        else:
            vol_position += trade.volume
            fiat_position -= trade.price_in_currency(price_currency)

        fiat_position -= trade.fee_in_currency(price_currency)

    return {volume_currency: vol_position, 'fiat': fiat_position}


def btc_position_delta(trades):
    """
    Get just the bitcoin position delta for a list of trades.
    """

    btc_position = Money('0', 'BTC')

    for trade in trades:
        if trade.trade_type == Consts.ASK:
            btc_position -= trade.volume
        else:
            btc_position += trade.volume

    return btc_position


def price_currency_for_trades(trades):
    """
    Simply takes a list of trades and extracts their currency.
    """

    if not trades:
        return "USD"

    currency = trades[0].price.currency

    # If our trades are in different currencies, convert them all to USD.
    if not all([(t.price.currency == currency) for t in trades]):
        currency = "USD"

    return currency



"""
This module was used for some configuration of the execution platform but is mostly
vestigial. It hasn't been removed yet because it is still referenced in the crypto
balancer module and also the 'fundamental values' library. Once those two libraries
are removed or rehabilitated this file should be removed and configurations should be
taken from .conf files.
"""

fv_v3_weights = {}

fv_v2_weights = {}

withdrawals_enabled = []

deposits_enabled = []

proportions = {}

account_value_limits = {}

forbidden_targets = {}




import pyximport; pyximport.install()

import os
import mock
import sure

from gryphon.lib.money import Money
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exchange_order import Order

from algos.multi import Multi

class TestLinear():

    ###              ORDER BOOK                ###
    ###       BIDS                 ASKS        ###
    ###       490                  510         ###
    ###       480                  520         ###
    ###       470                  530         ###
    ###               ... etc ...              ###

    def setUp(self):
        self.exchange = mock.Mock()
        self.exchange.name = "TestExchange"
        self.exchange.currency = "USD"

        self.db = mock.Mock()

        # Fundamental value calculations are tested in test_fundamental_value.py
        # so we can just hardcode it here
        self.fundamental_value = Money("500", "USD")

        self.orderbook = {'bids': [], 'asks': []}
        for i in range(1,20):
            bid = Order(Money("500", "USD") - (i * 10), Money("1", "BTC"), self.exchange, Order.BID, order_id=None, status=None)
            ask = Order(Money("500", "USD") + (i * 10), Money("1", "BTC"), self.exchange, Order.ASK, order_id=None, status=None)
            self.orderbook['bids'].append(bid)
            self.orderbook['asks'].append(ask)

        params = {
            'max_position': 'BTC 16',
            'volume': 'BTC 2',
            'max_fiat_balance': 'USD 200000',
            'jump': 'USD 0.01',
            'base_spread': 'USD 0',
            'ignore_volume': 'BTC 0.05',
        }

        self.algo = Multi(self.exchange, self.db, debug=True, params=params)
        self.algo._max_position = None
        self.algo._position = None
        self.algo.position = mock.Mock(return_value=Money(0, "BTC"))
        self.algo.order = mock.MagicMock()
        self.algo.update_order = mock.MagicMock()

    def test_calculate_price(self):
        bid_price, ask_price = self.algo.calculate_prices( self.fundamental_value , self.orderbook)

        bid_price.should.equal(Money("490.01", "USD"))
        ask_price.should.equal(Money("509.99", "USD"))

    def test_in_between_orders(self):
        self.algo.spread = Money("15.00", "USD")

        bid_price, ask_price = self.algo.calculate_prices( self.fundamental_value , self.orderbook)

        # we should ignore the tiny orders
        bid_price.should.equal(Money("480.01", "USD"))
        ask_price.should.equal(Money("519.99", "USD"))
        
    def test_ignore_tiny_orders(self):
        # very competitive orders in this order book
        tiny_bid = Order(Money("499", "USD"), Money("0.0001", "BTC"), self.exchange, Order.BID, order_id=None, status=None)
        tiny_ask = Order(Money("501", "USD"), Money("0.0001", "BTC"), self.exchange, Order.ASK, order_id=None, status=None)

        self.orderbook['bids'].insert(0, tiny_bid)
        self.orderbook['asks'].insert(0, tiny_ask)

        bid_price, ask_price = self.algo.calculate_prices( self.fundamental_value , self.orderbook)

        # we should ignore the tiny orders
        bid_price.should.equal(Money("490.01", "USD"))
        ask_price.should.equal(Money("509.99", "USD"))

    def test_should_not_bid_cross_the_spread(self):
        close_ask = Order(Money("490.01", "USD"), Money("1", "BTC"), self.exchange, Order.ASK, order_id=None, status=None)

        self.orderbook['asks'].insert(0, close_ask)

        bid_price, ask_price = self.algo.calculate_prices( self.fundamental_value , self.orderbook)

        # bid should not cross the spread
        # ask should be normal
        bid_price.should.equal(Money("490", "USD"))
        ask_price.should.equal(Money("509.99", "USD"))

    def test_should_not_ask_cross_the_spread(self):
        close_bid = Order(Money("509.99", "USD"), Money("1", "BTC"), self.exchange, Order.BID, order_id=None, status=None)

        self.orderbook['bids'].insert(0, close_bid)

        bid_price, ask_price = self.algo.calculate_prices( self.fundamental_value , self.orderbook)

        # bid should be normal
        # ask should not cross the spread
        bid_price.should.equal(Money("490.01", "USD"))
        ask_price.should.equal(Money("510", "USD"))

    def tearDown(self):
        pass



import pyximport; pyximport.install()
import gryphon.lib; gryphon.lib.prepare()

import datetime
import unittest

import mock
import numpy as np
import pandas as pd
import sure

import gryphon.lib.analysis.sharpe as sharpe


class TestSharpe(unittest.TestCase):
    def setUp(self):
        self.series = pd.Series(
            range(1, 24),
            index=[datetime.datetime(2015, 1, 1, i) for i in range(1,24)],
        )

    def tearDown(self):
        pass

    def test_basic(self):
        s = sharpe.calculate_sharpe_ratio(self.series, annualize_by_factor=24*365)
        s.should.equal(np.float64(89.671800634636767))

    def test_probabilistic(self):
        """
        Sharpe ratio of a normal(0,1) variable should be zero, since it's mean is
        close to zero and the std deviation is 1.
        """

        log_returns = pd.Series(np.random.normal(0, 1, 10000))
        s = sharpe.calculate_sharpe_ratio(log_returns)
        s = abs(s)

        self.assertTrue(s < 0.01)




import pyximport; pyximport.install()
import json
import os
import subprocess

from delorean import epoch
from raven import Client

import gryphon.data_service.consts as consts
from gryphon.data_service.queue_consumer import QueueConsumer
from gryphon.lib import session
from gryphon.lib.models.emeraldhavoc.orderbook import Orderbook


s = Client(dsn=os.environ.get('SENTRY_DSN'))


def orderbook_consumer_function(message, db):
    subprocess.call(["touch", "monit/heartbeat/orderbook_consumer.txt"])

    ob = json.loads(message)

    assert len(ob.keys()) == 2

    exchange_name = list(set(ob.keys()) - set(['timestamp'])).pop()
    timestamp = ob['timestamp']
    orderbook_data = ob[exchange_name]

    orderbook = Orderbook(
        exchange_name,
        orderbook=orderbook_data,
        timestamp=epoch(timestamp).datetime,
    )

    db.add(orderbook)
    session.commit_mysql_session(db)


def main():
    db = session.get_a_gds_db_mysql_session()

    try:
        orderbook_consumer = QueueConsumer(
            os.environ.get('AMPQ_ADDRESS'),
            orderbook_consumer_function,
            db,
            consts.EXCHANGE,
            consts.EXCHANGE_TYPE,
            consts.ORDERBOOK_BINDING_KEY,
            consts.ORDERBOOK_QUEUE,
        )

        orderbook_consumer.run()
    except KeyboardInterrupt:
        orderbook_consumer.stop()
    except:
        s.captureException()
    finally:
        db.remove()


if __name__ == '__main__':
    main()



"""
Exchange documentation: https://docs.gemini.com/rest-api/
"""
# -*- coding: utf-8 -*-
import base64
from collections import OrderedDict
import decimal
import hashlib
import hmac
import json
import time

from cdecimal import *

from gryphon.lib.exchange.exchange_api_wrapper import ExchangeAPIWrapper
from gryphon.lib.exchange import exceptions
from gryphon.lib.exchange import order_types
from gryphon.lib.models.exchange import Balance
from gryphon.lib.money import Money
from gryphon.lib.logger import get_logger

logger = get_logger(__name__)


class GeminiBTCUSDExchange(ExchangeAPIWrapper):
    def __init__(self, session=None, configuration=None):
        super(GeminiBTCUSDExchange, self).__init__(session)

        self.name = u'GEMINI_BTC_USD'
        self.friendly_name = u'Gemini BTC-USD'
        self.base_url = 'https://api.gemini.com/v1'
        self.currency = 'USD'
        self.volume_currency = 'BTC'
        self.bid_string = 'buy'
        self.ask_string = 'sell'

        self.gemini_pair_symbol = 'btcusd'

        # Configurables with defaults.
        self.market_order_fee = Decimal('0.0025')  # Updated by Gareth on 2016-9-20.
        self.limit_order_fee = Decimal('0.0000')
        self.fee = self.market_order_fee
        self.fiat_balance_tolerance = Money('0.0001', 'USD')
        self.volume_balance_tolerance = Money('0.00000001', 'BTC')
        self.max_tick_speed = 1
        self.min_order_size = Money('0.00001', 'BTC')
        self.use_cached_orderbook = False

        if configuration:
            self.configure(configuration)

    def get_ticker_req(self, verify=True):
        """
        Gemini has a ticker but it doesn't conform to our expected outputs. TODO decide
        what to do here.
        """
        pass

    def get_ticker_resp(self, req):
        return {
            'high': None,
            'low': None,
            'last': None,
            'volume': None,
        }

    def get_balance_req(self):
        return self.req('post', '/balances')

    def get_balance_resp(self, req):
        raw_balances = self.resp(req)

        btc_available = None
        usd_available = None

        for raw_balance in raw_balances:
            if raw_balance['currency'] == self.volume_currency:
                volume_currency_available = Money(
                    raw_balance['available'],
                    self.volume_currency,
                )
            elif raw_balance['currency'] == self.currency:
                price_currency_available = Money(
                    raw_balance['available'],
                    self.currency,
                )

        if volume_currency_available == None or price_currency_available == None:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'missing expected balances',
            )

        balance = Balance()
        balance[self.volume_currency] = volume_currency_available
        balance[self.currency] = price_currency_available

        return balance

    def _get_orderbook_from_api_req(self, verify=True):
        return self.req(
            'get',
            '/book/%s?limit_bids=0&limit_asks=0' % self.gemini_pair_symbol,
            no_auth=True,
            verify=verify,
        )

    @property
    def _orderbook_sort_key(self):
        return lambda o: float(o['price'])

    def parse_order(self, order):
        price = Money(order['price'], self.currency)
        volume = Money(order['amount'], self.volume_currency)

        return price, volume

    # Below this line is not good.

    def all_trades(self, until=None):
        req = self.all_trades_req(until)
        return self.all_trades_resp(req)

    def all_trades_req(self, until=None):
        payload = {
            'symbol': self.gemini_pair_symbol,
            'limit_trades': 500,
        }

        if until:
            payload['until'] = str(until)

        return self.req('post', '/mytrades', data=payload)

    def all_trades_resp(self, req):
        return self.resp(req)

    def trades_for_orders(self, order_ids):
        req = self.trades_for_orders_req()
        return self.trades_for_orders_resp(req, order_ids)

    def trades_for_orders_req(self):
        return self.all_trades_req()

    def trades_for_orders_resp(self, req, order_ids):
        order_ids = [unicode(o) for o in order_ids]
        trades = self.all_trades_resp(req)

        matching_trades = {}

        for trade in trades:
            oid = unicode(trade['order_id'])

            if oid in order_ids:
                if not oid in matching_trades:
                    matching_trades[oid] = []

                matching_trades[oid].append(trade)

        return matching_trades

    def place_order_req(self, mode, volume, price, order_type=order_types.LIMIT_ORDER):
        side = self._order_mode_from_const(mode)

        if price.currency not in ['USD', 'BTC']:
            raise ValueError('Invalid price currency %s' % price.currency)
        if volume.currency not in ['BTC', 'ETH']:
            raise ValueError('Invalid volume currency %s' % volume.currency)

        # TODO should this be rounding based on BID/ASK instead of always truncating?
        price_str = '%.2f' % price.amount
        volume_str = '%.8f' % volume.amount

        payload = {
            'symbol': self.gemini_pair_symbol,
            'amount': volume_str,
            'price': price_str,
            'side': side,
            'type': 'exchange limit',
        }

        return self.req('post', '/order/new', data=payload)

    def place_order_resp(self, req):
        response = self.resp(req)

        try:
            order_id = str(response['order_id'])

            return {'success': True, 'order_id': order_id}
        except KeyError:
            raise exceptions.ExchangeAPIErrorException(
                self, 
                'response does not contain an order id',
            )

    def get_open_orders_req(self):
        return self.req('post', '/orders')

    def get_open_orders_resp(self, req):
        raw_open_orders = self.resp(req)
        open_orders = []

        for raw_order in raw_open_orders:
            # Skip open orders on other pairs.
            if raw_order['symbol'] != self.gemini_pair_symbol:
                continue

            mode = self._order_mode_to_const(raw_order['side'])
            volume = Money(raw_order['original_amount'], self.volume_currency)

            volume_remaining = Money(
                raw_order['remaining_amount'],
                self.volume_currency,
            )

            order = {
                'mode': mode,
                'id': str(raw_order['id']),
                'price': Money(raw_order['price'], self.currency),
                'volume': volume,
                'volume_remaining': volume_remaining
            }

            open_orders.append(order)

        return open_orders

    def get_order_details(self, order_id):
        """
        Our implementation of the order_details functions is a little hacky, so we have
        to overload this function from the base class to make this function work.
        """
        req = self.get_order_details_req(order_id)
        return self.get_order_details_resp(req, order_id)

    def get_order_details_req(self, order_id):
        return self.get_multi_order_details_req()

    def get_order_details_resp(self, req, order_id):
        return self.get_multi_order_details_resp(req, [order_id])[order_id]

    def get_multi_order_details(self, order_id):
        """
        Same hack here as in get_order_details.
        """
        req = self.get_multi_order_details_req()
        return self.get_multi_order_details_resp(req, order_id)

    def get_multi_order_details_req(self):
        return self.trades_for_orders_req()

    def get_multi_order_details_resp(self, req, order_ids):
        order_ids = [unicode(o) for o in order_ids]

        multi_trades = self.trades_for_orders_resp(req, order_ids)
        data = {}

        for order_id in order_ids:
            total_price_currency = Money('0', self.currency)
            total_volume = Money('0', self.volume_currency)
            our_trades = []
            our_type = None

            if order_id in multi_trades:
                trades = multi_trades[order_id]

                for t in trades:
                    volume_amount = Money(t['amount'], self.volume_currency)
                    fee = Money(t['fee_amount'], t['fee_currency'])
                    price = Money(t['price'], self.currency)
                    price_currency_amount = price * volume_amount.amount
                    our_type = self._order_mode_to_const(t['type'].lower())

                    total_price_currency += price_currency_amount
                    total_volume += volume_amount

                    our_trades.append({
                        'time': int(float(t['timestamp'])),
                        'trade_id': unicode(t['tid']),
                        'fee': fee,
                        self.volume_currency.lower(): volume_amount,
                        'fiat': price_currency_amount,
                    })

            try:
                time_created = min([t['time'] for t in our_trades])
            except ValueError: # no trades
                time_created = None

            data[order_id] = {
                'time_created': time_created,
                'type': our_type,
                '%s_total' % self.volume_currency.lower(): total_volume,
                'fiat_total': total_price_currency,
                'trades': our_trades
            }

        return data

    def cancel_all_open_orders(self):
        """
        Gemini has it's own cancel-all-orders endpoint that doesn't require us to
        iterate through the list of open orders, so we override this.
        """
        req = self.cancel_all_open_orders_req()
        return self.cancel_all_open_orders_resp(req)

    def cancel_all_open_orders_req(self):
        return self.req('post', '/order/cancel/all')

    def cancel_all_open_orders_resp(self, req):
        resp = self.resp(req)

        if resp['result'] == 'ok':
            return {'success': True}
        else:
            return {'success': False}

    def cancel_order_req(self, order_id):
        payload = {
            'order_id': order_id,
        }

        return self.req('post', '/order/cancel', data=payload)

    def cancel_order_resp(self, req):
        response = self.resp(req)

        return {'success': True}

    def process_db_balance_for_audit(self, db_balance):
        # Gemini truncates BTC to 8 decimal places, but stores it to 10
        # and USD truncates to 2 decimal places, but is stored as 14
        if db_balance.currency == self.volume_currency:
            db_balance = db_balance.round_to_decimal_places(8, decimal.ROUND_FLOOR)
        elif db_balance.currency == self.currency:
            db_balance = db_balance.round_to_decimal_places(2, decimal.ROUND_FLOOR)

        return db_balance

    def get_order_audit_data(self, skip_recent=0):
        """
        Returns an OrderedDict of order ids mapped to their filled volume (including
        only orders that have some trades).
        """
        orders = OrderedDict()
        trades_to_audit = self.all_trades()

        # Trades from the same order aren't guaranteed to be next to each other, so we
        # need to group them.
        trades_to_audit.sort(key=lambda t: (t['order_id'], t['timestamp']), reverse=True)

        # We want to skip recent orders, but splitting between trades will give us
        # incorrect volume_filled.
        orders_to_skip = skip_recent

        while orders_to_skip > 0:
            try:
                trade = trades_to_audit.pop(0)

                # We found a boundary.
                if trade['order_id'] != trades_to_audit[0]['order_id']:
                    orders_to_skip -= 1
            except IndexError:
                # If we run off the end of the list, we are trying to strip all trades.
                trades_to_audit = []
                orders_to_skip = 0

        for trade in trades_to_audit:
            order_id = str(trade['order_id'])

            try:
                orders[order_id] += abs(Money(trade['amount'], self.volume_currency))
            except KeyError:
                orders[order_id] = abs(Money(trade['amount'], self.volume_currency))

        # Remove the oldest 2 orders, because its trades might be wrapped around a page
        # gap. This would give us an innacurate volume_filled number.
        # We need to remove 2, since there could be an ask and a bid.
        try:
            orders.popitem()
            orders.popitem()
        except KeyError:
            pass

        return orders

    def fiat_deposit_fee(self, deposit_amount):
        return Money('10', 'USD')

    # Request methods.

    def req(self, req_method, url, **kwargs):
        req = super(GeminiBTCUSDExchange, self).req(req_method, url, **kwargs)
        return req

    def resp(self, req):
        response = super(GeminiBTCUSDExchange, self).resp(req)

        if 'message' in response:
            errors_string = str(response['message'])

            if 'InsufficientFunds' in errors_string:
                raise exceptions.InsufficientFundsError()
            elif 'Order' in errors_string and 'not found' in errors_string:
                raise exceptions.CancelOrderNotFoundError()
            elif 'InvalidNonce' in errors_string:
                raise exceptions.NonceError()
            else:
                raise exceptions.ExchangeAPIErrorException(self, errors_string)

        return response

    def auth_request(self, req_method, url, request_args):
        try:
            self.api_key
            self.secret
        except AttributeError:
            self.api_key = self._load_env('GEMINI_BTC_USD_API_KEY')
            self.secret = self._load_env('GEMINI_BTC_USD_API_SECRET')

        endpoint = url.replace(self.base_url, '')
        endpoint = '/v1' + endpoint

        timestamp = int(round(time.time() * 1000))
        nonce = timestamp * 1000 + 1000000000

        try:
            data = request_args['data']
            # The payload get passed as a header, so we don't want to POST them.
            request_args['data'] = {}
        except KeyError:
            data = {}

        payload_object = {
            'request': endpoint,
            'nonce': str(nonce),
        }

        payload_object.update(data)
        payload_json = json.dumps(payload_object)
        payload = str(base64.b64encode(payload_json))

        sig = hmac.new(self.secret, payload, hashlib.sha384).hexdigest()

        try:
            headers = request_args['headers']
        except KeyError:
            headers = request_args['headers'] = {}

        headers['X-GEMINI-APIKEY'] = self.api_key
        headers['X-GEMINI-PAYLOAD'] = payload
        headers['X-GEMINI-SIGNATURE'] = sig

    # Helper functions and private methods.




# -*- coding: utf-8 -*-
from gryphon.data_service.pollers.trades.trades_poller import TradesPoller
from gryphon.lib.money import Money


class GeminiTrades(TradesPoller):
    def __init__(self):
        self.exchange_name = u'GEMINI_BTC_USD'
        self.url = 'https://api.gemini.com/v1/trades/BTCUSD?limit_trades=500'
        self.poll_time = 10

    def parse_trades(self, resp_obj):
        trades = []

        for trade in resp_obj:
            trade = {
                'exchange': self.exchange_name,
                'price': Money(trade['price'], 'USD'),
                'volume': Money(trade['amount'], 'BTC'),
                'timestamp': trade['timestamp'],
                'trade_id': trade['tid']
            }

            trades.append(trade)

        return trades



# -*- coding: utf-8 -*-
import os
from base import Base
import os
import json
import uuid
from datetime import datetime, timedelta
from sqlalchemy import ForeignKey, Column, Integer, Unicode, DateTime, UnicodeText, Numeric, desc
from sqlalchemy.orm import relationship, backref
from sqlalchemy.ext.declarative import declarative_base

from gryphon.lib import session
from gryphon.lib.money import Money

metadata = Base.metadata

class MarketDataRetriever(object):
    
    BITSTAMP=u'STMP'
    BTCE=u'BTCE'
    TRADE = u'TRADE'
    XBTUSD = u'XBT/USD'
    
    @staticmethod
    def trades(exchange=BITSTAMP, ticker_symbol=XBTUSD, start_time=datetime.utcnow()-timedelta(days=30), end_time=datetime.utcnow()):
        db = session.get_a_gds_db_mysql_session()
        trades = db.query(MarketData).filter_by(
            exchange=exchange).filter_by(
            ticker_symbol=ticker_symbol).filter_by(
            entry_type=u'TRADE').filter(
            MarketData.timestamp.between(start_time, end_time)).order_by(
            desc(MarketData.timestamp)).all()
        return trades
        

    def orderbook(self):
        pass
     


class MarketData(Base):
    __tablename__ = 'market_data'

    market_data_id = Column(Integer, primary_key=True)
    unique_id = Column(Unicode(64), nullable=False)
    time_added = Column(DateTime, nullable=False)

    timestamp = Column(DateTime, nullable=False)
    exchange = Column(Unicode(256))
    price = Column('price', Numeric(precision=20, scale=10))
    currency = Column(Unicode(256))
    volume = Column('volume', Numeric(precision=20, scale=10))
    entry_type = Column(Unicode(32), nullable=True)
    ticker_symbol = Column(Unicode(256))
    
    def __init__(self, timestamp, exchange, price, currency, volume, entry_type, ticker_symbol):
        self.unique_id = u'mkd_%s' % uuid.uuid4().hex
        self.time_added = datetime.utcnow()

        self.timestamp =timestamp
        self.exchange = exchange
        self.price = price
        self.currency = currency
        self.volume = volume
        self.entry_type = entry_type
        self.ticker_symbol = ticker_symbol
   
    def __str__(self):
        return unicode(self)
 
    def __unicode__(self):
        return '%s,%s,%s,%s,%s,%s,%s' % (
            self.timestamp,
            self.exchange,
            self.price,
            self.currency,
            self.volume,
            self.entry_type,
            self.ticker_symbol,
        )



from bitstamp import BitstampBTCUSDExchange
import os
import json
from cdecimal import *
from gryphon.lib.logger import get_logger
logger = get_logger(__name__)


class BitstampStagingExchange(BitstampBTCUSDExchange):
    def __init__(self, session=None, use_cached_orderbook=False):
        super(BitstampStagingExchange, self).__init__(session)
        self.name = u'BITSTAMP_STAGING'
        self.friendly_name = u'BitstampStaging'
        self.currency = u'USD'
        self.fee = Decimal("0.0020")
        self._order_book = None
        self.use_cached_orderbook = use_cached_orderbook

    def load_creds(self):
        try:
            self.api_key
            self.secret
            self.client_id
        except AttributeError:
            self.api_key = self.load_env('BITSTAMP_STAGING_API_KEY')
            self.secret = self.load_env('BITSTAMP_STAGING_API_SECRET')
            self.client_id = self.load_env('BITSTAMP_STAGING_CLIENT_ID')



from delorean import Delorean
from twisted.internet.task import LoopingCall
from twisted.python import log

from gryphon.data_service.auditors.auditor import Auditor


class PerformanceAuditor(Auditor):

    def start(self):
        self.time_between_calls = 10
        self.previous_timestamp = None
        self.looping_call = LoopingCall(self.audit)
        self.looping_call.start(self.time_between_calls)

    def audit(self):
        now = Delorean().epoch
        if not self.previous_timestamp:
            self.previous_timestamp = now
        else:
            expected_timestamp = self.previous_timestamp + self.time_between_calls
            actual_timestamp = now
            lag = actual_timestamp - expected_timestamp
            log.msg('Performance Lag: %.3fs' % lag)
            self.previous_timestamp = now



import pyximport; pyximport.install()

from gryphon.lib.exchange.quadriga_btc_cad import QuadrigaBTCCADExchange
from gryphon.tests.logic.exchange_wrappers.public_methods import ExchangePublicMethodsTests


class TestQuadrigaBTCCADPublicMethods(ExchangePublicMethodsTests):
    def setUp(self):
        self.exchange = QuadrigaBTCCADExchange()




# -*- coding: utf-8 -*-
from hashlib import sha256
import re
import phonenumbers
 
digits58 = '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'


# https://github.com/nederhoed/python-bitcoinadress/blob/master/bitcoinaddress/__init__.py
def _bytes_to_long(bytestring, byteorder):
    """For use in python version prior to 3.2 """
    result = []
    if byteorder == 'little':
        result = (v << i*8 for (i, v) in enumerate(bytestring))
    else:
        result = (v << i*8 for (i, v) in enumerate(reversed(bytestring)))
    return sum(result)

def _long_to_bytes(n, length, byteorder):
    """For use in python version prior to 3.2 
    Source:
    http://bugs.python.org/issue16580#msg177208
    """
    if byteorder == 'little':
        indexes = range(length)
    else:
        indexes = reversed(range(length))
    return bytearray((n >> i*8) & 0xff for i in indexes)

def decode_base58(bitcoin_address, length):
    """Decode the base58 encoded address. 
    This form of base58 decoding is Bitcoin specific. 
    Be careful outside of Bitcoin context.
    """
    n = 0
    for char in bitcoin_address:
        n = n * 58 + digits58.index(char)
    try:
        return n.to_bytes(length, 'big')
    
    except AttributeError:
        # Python version < 3.2
        return _long_to_bytes(n, length, 'big')

def encode_base58(bytestring):
    """Encode the bytestring to a base58 encode string. """
    # Count zero's
    zeros = 0
    for i in range(len(bytestring)):
        if bytestring[i] == 0:
            zeros += 1
        else:
            break
    try:
        n = int.from_bytes(bytestring, 'big')
    except AttributeError:
        # Python version < 3.2
        n = _bytes_to_long(bytestring, 'big')
    result = ''
    (n, rest) = divmod(n, 58)
    while n or rest:
        result += digits58[rest]
        (n, rest) = divmod(n, 58)
    return zeros * '1' + result[::-1]  # reverse string
 
def is_valid_btc_address(bitcoin_address):
    """Check the integrity of the bitcoin address.
    
    Returns False if the address is invalid.
    >>> validate('1AGNa15ZQXAZUgFiqJ2i7Z2DPU2J6hW62i')
    True
    >>> validate('')
    False
    """
    try:
        bcbytes = decode_base58(bitcoin_address, 25)
        # Compare checksum
        checksum = sha256(sha256(bcbytes[:-4]).digest()).digest()[:4]
        if bcbytes[-4:] != checksum:
            return False
        # Encoded bytestring should be equal to the original address
        # For example '14oLvT2' has a valid checksum, but is not a valid btc address
        return bitcoin_address == encode_base58(bcbytes)
    except:
        return False

def is_valid_email(email):
    if re.match(r"[^@]+@[^@]+\.[^@]+", email):
        return True
    else:
        return False
    
def is_valid_phone_number(number_str):
    try:
        z = phonenumbers.parse(number_str, 'US')
        return phonenumbers.is_valid_number(z)
    except:
        return False
    
     



import pyximport; pyximport.install()

import unittest

from gryphon.lib.exchange.exchange_order import Order as ExchangeOrder
from gryphon.lib.metrics import volume_available
from gryphon.lib.models.order import Order
from gryphon.lib.money import Money


class TestLiquidityFunction(unittest.TestCase):
    def setUp(self):
        self.empty_book = {
            'bids': [],
            'asks': [],
        }

        self.exchange = None

        bid = ExchangeOrder(
            Money('249', 'USD'), 
            Money('10', 'BTC'),
            self.exchange,
            Order.BID,
        )
            
        ask = ExchangeOrder(
            Money('251', 'USD'), 
            Money('10', 'BTC'),
            self.exchange,
            Order.ASK,
        )

        self.basic_book = {
            'bids': [bid],
            'asks': [ask],
        }

        bid2 = ExchangeOrder(
            Money('239', 'USD'),
            Money('10', 'BTC'),
            self.exchange,
            Order.BID,
        )

        ask2 = ExchangeOrder(
            Money('261', 'USD'),
            Money('100', 'BTC'),
            self.exchange,
            Order.ASK,
        )

        self.bigger_book = {
            'bids': [bid, bid2],
            'asks': [ask, ask2],
        }

    def test_empty(self):
        did_throw_exception = False

        try:
            liquidity = volume_available.volume_available_at_price(
                Order.BID,
                Money('250', 'USD'),
                self.empty_book,
            )
        except Exception:
            did_throw_exception = True            

        did_throw_exception.should.equal(True)

    def test_no_overlap(self):
        liquidity = volume_available.volume_available_at_price(
            Order.ASK,
            Money('250', 'USD'),
            self.basic_book,
        )

        liquidity.should.equal(Money('0', 'BTC'))




"""
This sub-strategy gives code to maintain backstop orders in a market making algorithm
far back of the midpoint (e.g., 10% back of the midpoint) in order to catch black swan
swings that we can use for arbitrage or simply to open positions at very favourable
prices.
"""

from datetime import datetime, timedelta

from cdecimal import *
import termcolor as tc

from gryphon.execution.strategies.base import Strategy
from gryphon.lib.exchange.base import Exchange
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exceptions import *
from gryphon.lib.models.datum import Datum
from gryphon.lib.models.event import EventRecorder
from gryphon.lib.money import *
from gryphon.lib.session import get_a_redis_connection


ABSOLUTE_MAX_ORDER_VOLUME = Money('128', "BTC")
PREDICTIVE_VOLUME_REDIS_KEY = 'ML_PREDICTION'


class BackstopOrdersStrategy(Strategy):
    def should_use_backstop_orders(self):
        backstops_enabled = self.config.get('use_backstop_orders', 'no') == 'yes'

        can_place_orders = (
            self.has_required_balance_for_backstop_bid()
            or self.has_required_balance_for_backstop_ask()
        )

        if backstops_enabled and can_place_orders:
            return True
        else:
            return False

    def has_required_balance_for_backstop_bid(self):
        """
        We require enough fiat balance to place a 64 bitcoin buy order and a 10 bitcoin
        backstop in order still be placing backstop orders.
        """

        max_bids_total_volume = (
            self.config['max_position'] + self.config['backstop_volume']
        )

        required_balance = max_bids_total_volume.amount * self.fundamental_value

        if self.exchange_data.balance[self.exchange.currency] > required_balance:
            return True
        else:
            return False

    def has_required_balance_for_backstop_ask(self):
        """
        We require enough bitcoin balance to place a 64 bitcoin sell order and a 10
        bitcoin backstop in order still be placing backstop orders.
        """

        required_balance = self.config['max_position'] + self.config['backstop_volume']

        if self.exchange_data.balance['BTC'] > required_balance:
            return True
        else:
            return False

    def current_backstop_bid_and_ask(self, current_orders):
        try:
            bids = [
                o for o in current_orders
                if o['mode'] == Consts.BID and o['is_backstop']== True
            ]

            assert len(bids) <= 1

            bid = bids[0]
        except IndexError:
            bid = None

        try:
            asks = [
                o for o in current_orders
                if o['mode'] == Consts.ASK and o['is_backstop'] == True
            ]

            assert len(asks) <= 1

            ask = asks[0]
        except IndexError:
            ask = None

        return bid, ask

    def current_core_bid_and_ask(self, current_orders):
        try:
            bids = [
                o for o in current_orders
                if o['mode'] == Consts.BID and o['is_backstop'] == False
            ]

            assert len(bids) <= 1

            bid = bids[0]
        except IndexError:
            bid = None

        try:
            asks = [
                o for o in current_orders
                if o['mode'] == Consts.ASK and o['is_backstop'] == False
            ]

            assert len(asks) <= 1

            ask = asks[0]
        except IndexError:
            ask = None

        return (bid, ask)

    def calculate_backstop_volumes(self):
        """
        The current sizing logic for backstops is to have a fixed size order if we
        pass the has_required_required_balance checks and zero otherwise.
        """
        backstop_bid_volume = Money('0', 'BTC')
        backstop_ask_volume = Money('0', 'BTC')

        if self.has_required_balance_for_backstop_bid():
            backstop_bid_volume = self.config['backstop_volume']

        if self.has_required_balance_for_backstop_ask():
            backstop_ask_volume = self.config['backstop_volume']

        return backstop_bid_volume, backstop_ask_volume

    def calculate_backstop_prices(self, core_bid_price, core_ask_price):
        backstop_ask_price = core_ask_price + self.config['backstop_price_level']
        backstop_bid_price = core_bid_price - self.config['backstop_price_level']

        return backstop_bid_price, backstop_ask_price

    def backstop_datums(self):
        return [Datum('BACKSTOP')]

    def cancel_all_backstop_orders(self, current_orders):
        current_bid, current_ask = self.current_backstop_bid_and_ask(current_orders)

        if current_bid:
            self.harness.cancel_order(current_bid['id'])

        if current_ask:
            self.harness.cancel_order(current_ask['id'])

        return

    def update_backstop_orders(self, core_bid_price, core_ask_price, orderbook, current_orders):
        """
        Set a pair of orders far back back on either side of the fundamental value in
        order to catch a very fast swing.
        """

        if not self.should_use_backstop_orders():
            self.cancel_all_backstop_orders(current_orders)
            return

        current_bid, current_ask = self.current_backstop_bid_and_ask(current_orders)

        # Cancel any existing backstop bid if we've just moved into a state where
        # has_required_balance no longer passes.
        if not self.has_required_balance_for_backstop_bid():
            if current_bid:
                self.harness.cancel_order(current_bid['id'])

        # Likewise for asks.
        if not self.has_required_balance_for_backstop_ask():
            if current_ask:
                self.harness.cancel_order(current_ask['id'])

        backstop_bid_volume, backstop_ask_volume = self.calculate_backstop_volumes()

        backstop_bid_price, backstop_ask_price = self.calculate_backstop_prices(
            core_bid_price,
            core_ask_price,
        )

        backstop_datums = self.backstop_datums()

        if self.config.get('buy', 'yes') != 'no':
            try:
                if not current_bid:
                    self.log(
                        'Placing new Backstop_bid: %s at %s',
                        (backstop_bid_volume, backstop_bid_price),
                    )

                    self.harness.order(
                        Consts.BID,
                        backstop_bid_volume,
                        backstop_bid_price,
                        orderbook=orderbook,
                        datums=backstop_datums,
                    )

                else:
                    current_bid_price = current_bid['price']

                    if self.are_different_enough(current_bid_price, backstop_bid_price, self.config['backstop_order_ignore_price']) or self.are_different_enough(current_bid['volume_remaining'], backstop_bid_volume, self.config['order_ignore_volume']):

                        self.log(
                            'Updating backstop bid, %s is different enough than %s',
                            (current_bid_price, backstop_bid_price),
                            'green',
                        )

                        self.harness.update_order(
                            current_bid,
                            backstop_bid_price,
                            backstop_bid_volume,
                            orderbook=orderbook,
                            datums=backstop_datums,
                        )
                    else:
                        self.log(
                            'Not updating backstop bid, %s is not different enough than %s',
                            (current_bid_price, backstop_bid_price),
                        )

            except InsufficientFundsError:
                self.log('InsufficientFundsError, skipping', color='red')

        if self.config.get('sell', 'yes') != 'no':
            try:
                if not current_ask:
                    self.log(
                        'Placing new Backstop Ask: %s at %s',
                        (backstop_ask_volume, backstop_ask_price),
                    )

                    self.harness.order(
                        Consts.ASK,
                        backstop_ask_volume,
                        backstop_ask_price,
                        orderbook=orderbook,
                        datums=backstop_datums,
                    )

                else:
                    current_ask_price = current_ask['price']

                    if self.are_different_enough(current_ask_price, backstop_ask_price, self.config['backstop_order_ignore_price']) or self.are_different_enough(current_ask['volume_remaining'], backstop_ask_volume, self.config['order_ignore_volume']):

                        self.log(
                            'Updating backstop ask, %s is different enough than %s',
                            (current_ask_price, backstop_ask_price),
                            'green',
                        )

                        self.harness.update_order(
                            current_ask,
                            backstop_ask_price,
                            backstop_ask_volume,
                            orderbook=orderbook,
                            datums=backstop_datums,
                        )
                    else:
                        self.log(
                            'Not updating backstop ask, %s is not different enough than %s',
                            (current_ask_price, backstop_ask_price),
                        )
            except InsufficientFundsError:
                self.log('InsufficientFundsError, skipping', color='red')




import numpy as np

def rsiFunc(prices, n=14):
    deltas = np.diff(prices)
    seed = deltas[:n+1]
    up = seed[seed>=0].sum()/n
    down = -seed[seed<0].sum()/n
    rs = up/down
    rsi = np.zeros_like(prices)
    rsi[:n] = 100. - 100./(1.+rs)

    for i in range(n, len(prices)):
        delta = deltas[i-1] # cause the diff is 1 shorter

        if delta>0:
            upval = delta
            downval = 0.
        else:
            upval = 0.
            downval = -delta

        up = (up*(n-1) + upval)/n
        down = (down*(n-1) + downval)/n

        rs = up/down
        rsi[i] = 100. - 100./(1.+rs)

    return rsi



import os
import subprocess
import time

import termcolor as tc

from gryphon.execution.lib.heartbeat import heartbeat
from gryphon.execution.bots import shoebox
from gryphon.lib import session
from gryphon.lib.exchange import exchange_factory
from gryphon.lib.logger import get_logger
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.scrapers.base import Scraper
from gryphon.lib.scrapers.bmo import BMOScraper
from gryphon.lib.scrapers.boa import BoAScraper
from gryphon.lib.slacker import Slacker
from gryphon.lib.util.time import humanize_seconds

logger = get_logger(__name__)

TICK_SLEEP = 60 * 60  # 1 hour
BURN_THRESHOLD = 10000

BANK_AUDIT_HEARTBEAT_KEY = 'BANK_AUDIT_%s'


def run():
    db = session.get_a_trading_db_mysql_session()

    try:
        logger.info('Reporting for duty.')

        while True:
            audit_bmo_accounts(db)
            audit_boa_accounts(db)

            session.commit_mysql_session(db)
            logger.info('Going to sleep for %s.' % humanize_seconds(TICK_SLEEP))
            time.sleep(TICK_SLEEP)

    finally:
        db.remove()


def audit_bmo_accounts(db):
    scraper = BMOScraper()
    audit_bank_accounts(db, scraper)
    scraper.quit()


def audit_boa_accounts(db):
    scraper = BoAScraper()
    audit_bank_accounts(db, scraper)
    scraper.quit()


def audit_bank_accounts(db, scraper):
    """
    Check the balances of each bank account. If they differ, look for and add the new transactions.

    Writes BANK_AUDIT_<key> heartbeats on success.
    """
    try:
        account_data = scraper.load()
    except Scraper.MaintenanceException as e:
        # The heartbeats will fail, but the audit will retry every hour.
        scraper_name = scraper.__class__.__name__
        logger.info(tc.colored('%s: Service Unavailable' % scraper_name, 'yellow'))

        msg = str(e)
        if msg:
            logger.info(tc.colored(msg, 'yellow'))

        return

    for account in account_data:
        account_num = account['account_number']
        account_key = account_num_to_key(account_num)

        db_account = exchange_factory.make_exchange_data_from_key(account_key, db)

        db_balance = db_account.balance.fiat()
        ledger_balance = db_account.ledger_balance().fiat()

        if ledger_balance != db_balance:
            msg = 'DB balance: %s != Ledger balance: %s' % (db_balance, ledger_balance)
            logger.info(tc.colored(msg, 'red'))
            return

        bank_balance = account['balance']

        if bank_balance == db_balance:
            success(account_key)
        else:
            balance_diff = bank_balance - db_balance
            transactions = scraper.load_transactions(account_num)

            new_transactions = find_new_transactions(transactions, balance_diff)
            record_transactions(db_account, new_transactions, db)

            db_balance = db_account.balance.fiat()
            if bank_balance == db_balance:
                success(account_key)
            else:
                msg = 'Our balance: %s != Bank balance: %s' % (db_balance, bank_balance)
                logger.info(tc.colored(msg, 'red'))


def find_new_transactions(transactions, balance_diff):
    """
    Find the latest n transactions which caused our balance to change by <balance_diff>.
    """
    if balance_diff.amount == 0:
        return []

    logger.debug('looking for new transactions to match a %s diff' % balance_diff)

    running_total = 0
    new_transactions = []
    for transaction in reversed(transactions):
        amount = transaction['amount']
        if transaction['type'] == Transaction.WITHDRAWL:
            amount = -amount

        running_total += amount

        logger.debug('found a transaction for %s' % amount)
        new_transactions.append(transaction)

        # TODO: Once we have been running the bank auditor for a bit and have good data,
        # we probably also want to make sure that the most recent db transaction matches the
        # next oldest transaction from BMO.
        #
        # most_recent_db_transaction = bmo_usd_db.transactions\
        #     .filter(Transaction.transaction_status == Transaction.COMPLETED)\
        #     .order_by(Transaction.time_completed.desc())\
        #     .first()
        # if do_transactions_match(most_recent_db_transaction, transaction)

        if running_total == balance_diff:
            return new_transactions

    return []


def do_transactions_match(db_transaction, bank_transaction):
    """
    Check if a transaction from our database matches a transaction from the bank scraper.
    """
    return (
        db_transaction.transaction_details['description'] == bank_transaction['description'] and
        db_transaction.transaction_type == bank_transaction['type'] and
        db_transaction.amount == bank_transaction['amount']
    )


def record_transactions(db_account, transactions, db):
    """
    Record any new transactions found in our bank account.

    This fall into 3 categories:
    1. Expected pending deposits (wires from exchanges)
    2. Unexpected deposits (reverse burn)
    3. Unexpected withdrawals (burn)

    Expected withdrawals (wires to exchanges) are handled by manual withdrawals
    done from the command line by the person doing money-moving.
    """
    for transaction in transactions:
        if transaction['type'] == Transaction.DEPOSIT:
            pending_deposit = db_account.deposit_landed(transaction['amount'])
            if pending_deposit:
                logger.info(tc.colored('Recorded pending deposit for %s [%s]' % (transaction['amount'], transaction['description']), 'green'))
                notify_on_call_dev(transaction, db_account)
                continue
            elif transaction['amount'] < BURN_THRESHOLD:
                record_reverse_burn(db_account, transaction, db)
                logger.info(tc.colored('Recorded reverse burn for %s [%s]' % (transaction['amount'], transaction['description']), 'green'))
        elif transaction['type'] == Transaction.WITHDRAWL:
            # Sanity check (burn transactions should be under $10,000)
            if transaction['amount'] < BURN_THRESHOLD:
                record_burn(db_account, transaction, db)
                logger.info(tc.colored('Recorded burn for %s [%s]' % (transaction['amount'], transaction['description']), 'green'))


def notify_on_call_dev(transaction, db_account):
    """
    Notify the current on-call dev that a wire has landed
    """
    try:
        on_call_dev = shoebox.get_on_call_dev()
        channel = '@%s' % on_call_dev

        message = '%s wire landed in %s [%s]' % (transaction['amount'], db_account.name, transaction['description'])

        slacker = Slacker(channel, 'mover', icon_emoji=':moneybag:')
        slacker.notify(message)
    except:
        pass


def record_burn(db_account, transaction, db):
    """
    Record transactions which "unexpectedly" took money out of our bank account.

    We expect this for burn: payroll, rent, bills, etc.
    """
    if transaction['type'] != Transaction.WITHDRAWL:
        raise ValueError('Burn must be a withdrawal')

    amount = transaction['amount']

    transaction_details = {}
    transaction_details['description'] = transaction['description']

    burn_account = exchange_factory.make_exchange_data_from_key('BURN', db)

    deposit, __ = db_account.record_fiat_withdrawal(
        burn_account,
        amount,
        transaction_details=transaction_details,
    )
    deposit.complete()


def record_reverse_burn(db_account, transaction, db):
    """
    Record transactions which "unexpectedly" put money back into our bank account.

    We expect this for interest, Matinee payments, reversed bank errors, etc.
    """
    if transaction['type'] != Transaction.DEPOSIT:
        raise ValueError('Reverse burn must be a deposit')

    amount = transaction['amount']

    transaction_details = {}
    transaction_details['description'] = transaction['description']

    burn_account = exchange_factory.make_exchange_data_from_key('BURN', db)

    deposit, __ = burn_account.record_fiat_withdrawal(
        db_account,
        amount,
        transaction_details=transaction_details,
    )
    deposit.complete()


def account_num_to_key(account_num):
    """
    Convert from BMO account number to exchange database key.

    eg: '2423 xxxx-xxx' -> 'BMO_USD'
    Requires the <KEY>_ACCOUNT_NUMBER env vars to be set.
    """
    accounts_map = load_accounts_map()
    reverse_map = {num: key for key, num in accounts_map.items()}

    return reverse_map[account_num]


def key_to_account_num(key):
    """
    Convert from exchange database key to BMO account number.

    eg: 'BMO_USD' -> '2423 xxxx-xxx'
    Requires the <KEY>_ACCOUNT_NUMBER env vars to be set.
    """
    accounts_map = load_accounts_map()

    return accounts_map[key]


def load_accounts_map():
    accounts_map = {}

    for key in exchange_factory.BANK_ACCOUNT_KEYS:
        env_name = '%s_ACCOUNT_NUMBER' % key
        account_num = os.environ[env_name]
        accounts_map[key] = account_num

    return accounts_map


def success(name):
    logger.info(tc.colored('%s passed the audit' % name, 'green'))
    heartbeat(BANK_AUDIT_HEARTBEAT_KEY % name)



# -*- coding: utf-8 -*-
from gryphon.data_service.auditors.bitstamp_orderbook_auditor import BitstampOrderbookAuditor
from gryphon.data_service.auditors.coinbase_cad_orderbook_auditor import CoinbaseCADOrderbookAuditor
from gryphon.data_service.auditors.coinbase_orderbook_auditor import CoinbaseOrderbookAuditor
from gryphon.data_service.auditors.heartbeat_auditor import HeartbeatAuditor
from gryphon.data_service.auditors.performance_auditor import PerformanceAuditor
from gryphon.data_service.auditors.trades_auditor import TradesAuditor
from gryphon.data_service.task import Task


class AuditorTask(Task):
    def __init__(self, exchanges=[]):
        exchange_orderbook_auditors = {
            'COINBASE_BTC_USD': CoinbaseOrderbookAuditor(),
            'BITSTAMP_BTC_USD': BitstampOrderbookAuditor(),
            'COINBASE_BTC_CAD': CoinbaseCADOrderbookAuditor(),
        }

        if exchanges:
            self.auditors = [
                exchange_orderbook_auditors[ex]
                for ex in exchanges
                if ex in exchange_orderbook_auditors
            ]

        else:
            self.auditors = exchange_orderbook_auditors.values()

        self.auditors.extend([
            HeartbeatAuditor(),
            PerformanceAuditor(),
            TradesAuditor(exchanges=exchanges)
        ])

    def start_task(self):
        for auditor in self.auditors:
            auditor.start()




import pyximport; pyximport.install()
import gryphon.lib; gryphon.lib.prepare()

from decimal import Decimal
import mock
import sure
import unittest

from gryphon.lib.exchange.consts import Consts
from gryphon.lib.gryphonfury import positions
from gryphon.lib.gryphonfury import revenue as revenue_lib
from gryphon.lib.models.trade import Trade
from gryphon.lib.money import Money


class TestProfit():
    def setUp(self):
        self.order = mock.MagicMock()
        self.order.exchange_rate = Decimal('0.80')
        self.order.fundamental_value = Money('250', 'USD')

        self.trades = []

        self.bid = Trade(
            Consts.BID,
            Money('100', 'USD'),
            Money('0', 'USD'),
            Money('1', 'BTC'),
            '1',
            self.order,
        )

        self.trades.append(self.bid)

        self.ask = Trade(
            Consts.ASK,
            Money('100', 'USD'),
            Money('0', 'USD'),
            Money('1', 'BTC'),
            '2',
            self.order,
        )

        self.trades.append(self.ask)

    def tearDown(self):
        pass

    def test_position_delta(self):
        position = positions.position_delta(self.trades)

        position['BTC'].should.equal(Money('0', 'BTC'))
        position['fiat'].should.equal(Money('0', 'USD'))

    def test_position_delta_with_fees(self):
        self.bid.fee = Money('1', 'USD')
        self.ask.fee = Money('2', 'USD')

        position = positions.position_delta(self.trades)

        position['BTC'].should.equal(Money('0', 'BTC'))
        position['fiat'].should.equal(Money('-3', 'USD'))

    def test_position_delta_with_btc_fees(self):
        self.bid.fee = Money('0.01', 'BTC')
        self.ask.fee = Money('0.02', 'BTC')

        position = positions.position_delta(self.trades)

        position['BTC'].should.equal(Money('0', 'BTC'))
        position['fiat'].should.equal(Money('-7.5', 'USD'))

    def test_split_trades(self):
        matched_trades, position_trades = revenue_lib.split_trades(self.trades)

        # Expect to match all of #1 with #2 with no remaining position.
        matched_trades.should.have.length_of(2)
        matched_trades[0].volume.should.equal(Money('1', 'BTC'))
        matched_trades[1].volume.should.equal(Money('1', 'BTC'))

        position_trades.should.be.empty

    def test_split_trades_partial_matches(self):
        self.ask.volume = Money('0.25', 'BTC')

        new_ask = Trade(
            Consts.ASK,
            Money('100', 'USD'),
            Money('0', 'USD'),
            Money('2', 'BTC'),
            '3',
            self.order,
        )

        new_ask._exchange_rate = None
        self.trades.append(new_ask)
        matched_trades, position_trades = revenue_lib.split_trades(self.trades)

        # expect to match:
        # 0.25 of #1 with all of #2
        # 0.75 of #1 with 0.75 of #3
        # with 1.25 of #3 left in position
        matched_trades.should.have.length_of(4)
        matched_trades[0].volume.should.equal(Money('0.25', 'BTC'))
        matched_trades[1].volume.should.equal(Money('0.25', 'BTC'))
        matched_trades[2].volume.should.equal(Money('0.75', 'BTC'))
        matched_trades[3].volume.should.equal(Money('0.75', 'BTC'))

        position_trades.should.have.length_of(1)
        position_trades[0].volume.should.equal(Money('1.25', 'BTC'))

    def test_split_trades_with_btc_fees(self):
        self.bid.fee = Money('0.01', 'BTC')

        matched_trades, position_trades = revenue_lib.split_trades(self.trades)

        # expect to match all of #1 with #2
        # with no remaining position
        matched_trades.should.have.length_of(2)
        matched_trades[0].volume.should.equal(Money('1', 'BTC'))
        matched_trades[1].volume.should.equal(Money('1', 'BTC'))

        position_trades.should.be.empty

    def test_profit_units(self):
        self.bid.price = Money('90', 'USD')
        matched_trades, position_trades = revenue_lib.split_trades(self.trades)
        profit_units = revenue_lib.profit_units(matched_trades)
        profit_units.should.have.length_of(1)
        profit_units[0]['profit'].should.equal(Money('10', 'USD'))

    def test_profit_units_with_btc_fees(self):
        self.bid.fee = Money('0.01', 'BTC')
        matched_trades, position_trades = revenue_lib.split_trades(self.trades)
        profit_units = revenue_lib.profit_units(matched_trades)

        profit_units.should.have.length_of(1)
        profit_units[0]['profit'].should.equal(Money('-2.50', 'USD'))

    def test_profit_units_with_btc_and_fiat_fees(self):
        self.bid.fee = Money('0.01', 'BTC')
        self.ask.fee = Money('1', 'USD')

        matched_trades, position_trades = revenue_lib.split_trades(self.trades)
        profit_units = revenue_lib.profit_units(matched_trades)

        profit_units.should.have.length_of(1)
        profit_units[0]['profit'].should.equal(Money('-3.50', 'USD'))

    def test_profit_units_with_btc_and_fiat_fees_and_cad(self):
        self.bid.volume = Money('1', 'BTC')
        self.bid.price = Money('100', 'CAD')
        self.bid.fee = Money('0.01', 'BTC')

        self.ask.volume = Money('1', 'BTC')
        self.ask.price = Money('101', 'CAD')
        self.ask.fee = Money('1', 'CAD')
        
        self.order.fundamental_value = Money(250, 'CAD')
        
        matched_trades, position_trades = revenue_lib.split_trades(self.trades)
        profit_units = revenue_lib.profit_units(matched_trades)
        profit_units.should.have.length_of(1)
        profit_units[0]['profit'].should.equal(Money('-2.50', 'CAD'))
        
    def test_profit_with_btc_and_fiat_fees_and_cad(self):
        self.bid.volume = Money('1', 'BTC')
        self.bid.price = Money('100', 'CAD')
        self.bid.fee = Money('0.01', 'BTC')

        self.ask.volume = Money('1', 'BTC')
        self.ask.price = Money('101', 'CAD')
        self.ask.fee = Money('1', 'CAD')
        
        self.order.fundamental_value = Money(250, 'CAD')
    
        matched_trades, position_trades = revenue_lib.split_trades(self.trades)
        p = revenue_lib.realized_pl(matched_trades)

        p.should.equal(Money('-2.50', 'CAD'))




# -*- coding: utf-8 -*-
import os

import tornado.web

from gryphon.dashboards.handlers.assets import AssetsHandler
from gryphon.dashboards.handlers.balances import AllBalancesHandler
#from gryphon.dashboards.handlers.block_times import BlockTimesHandler
from gryphon.dashboards.handlers.fees import FeesDashboardHandler
from gryphon.dashboards.handlers.home import HomeHandler
from gryphon.dashboards.handlers.ledger import LedgerHandler
from gryphon.dashboards.handlers.login import LoginHandler
from gryphon.dashboards.handlers.logout import LogoutHandler
from gryphon.dashboards.handlers.notfound import NotFoundHandler
from gryphon.dashboards.handlers.orderbooks.orderbook import OrderbooksHandler
from gryphon.dashboards.handlers.status import GryphonStatusHandler
#from gryphon.dashboards.handlers.tick_times import TickTimesHandler
from gryphon.dashboards.handlers.trade_view import TradeViewHandler
from gryphon.dashboards.handlers.strategies.strategy import StrategyTradingHandler

ROOT = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
APP_ROOT = os.environ['APP_ROOT']
SRC_ROOT = os.path.join(ROOT, APP_ROOT)


url_patterns = [
    (r"/?", HomeHandler),
    (r"/404", NotFoundHandler),
    (r"/balances/?", AllBalancesHandler),
    (r"/assets/?", AssetsHandler),
    (r"/status/?", GryphonStatusHandler),
    (r"/fees/?", FeesDashboardHandler),
#    (r"/ticktimes/?", TickTimesHandler),
#    (r"/block_times/(.+)/?", BlockTimesHandler),
    (r"/orderbooks/(.+?)/?", OrderbooksHandler),
    (r"/ledger/(.+?)(?:/(.+?))?/?", LedgerHandler),
    (r"/strategies/(.+?)/?", StrategyTradingHandler),
    (r"/tradeview/?", TradeViewHandler),
    (r"/login/?", LoginHandler),
    (r"/logout/?", LogoutHandler),
    (r"/static/(.*)", tornado.web.StaticFileHandler, {'path': '%s/static' % SRC_ROOT}),
]



# -*- coding: utf-8 -*-
import json
import os

from gryphon.lib.session import get_a_redis_connection
from delorean import Delorean, epoch
import requests

#needs search query, start_time, end_time and score.
hn_url = "http://hn.algolia.com/api/v1/search_by_date?"\
            "query=%s&"\
            "tags=story&"\
            "hitsPerPage=500&"\
            "numericFilters=created_at_i>%s,points>=%s"

story_list_key = 'news_already_seen'


def get_hn_breaking_news(keywords={'bitcoin': 3, 'bitstamp': 0}):
    # keywords dictionary represents a search term paired with the threshold
    # for how many upvotes results for that term need to be considered breaking news
    try:
        redis = get_a_redis_connection(creds=os.environ['NOODLE_DB_REDIS'])

        news_already_seen = json.loads(redis.get(story_list_key) or '[]')
        breaking_news = []

        one_day_ago = Delorean().last_day(1).epoch

        for keyword, upvote_threshold in keywords.iteritems():
            # We quote the keywords so they don't match things that are close
            # Prompted by Bitcoin matching Ditchin (https://news.ycombinator.com/item?id=10850368)
            keyword = '"%s"' % keyword
            final_hn_url = hn_url % (keyword, one_day_ago, upvote_threshold)
            response = requests.get(final_hn_url).json()
            submissions = response['hits']

            for s in submissions:
                if s['url'] and s['url'] not in news_already_seen:
                    comments_url = 'https://news.ycombinator.com/item?id=%s' % s['objectID']
                    news = {'title': s['title'], 'url': s['url'], 'comments_url': comments_url}
                    breaking_news.append(news)
                    news_already_seen.append(s['url'])

        redis.set(story_list_key, json.dumps(news_already_seen))
        return breaking_news

    finally:
        redis.connection_pool.disconnect()
